var APP = (function () {
	'use strict';

	var isMergeableObject = function isMergeableObject(value) {
		return isNonNullObject(value)
			&& !isSpecial(value)
	};

	function isNonNullObject(value) {
		return !!value && typeof value === 'object'
	}

	function isSpecial(value) {
		var stringValue = Object.prototype.toString.call(value);

		return stringValue === '[object RegExp]'
			|| stringValue === '[object Date]'
			|| isReactElement(value)
	}

	// see https://github.com/facebook/react/blob/b5ac963fb791d1298e7f396236383bc955f916c1/src/isomorphic/classic/element/ReactElement.js#L21-L25
	var canUseSymbol = typeof Symbol === 'function' && Symbol.for;
	var REACT_ELEMENT_TYPE = canUseSymbol ? Symbol.for('react.element') : 0xeac7;

	function isReactElement(value) {
		return value.$$typeof === REACT_ELEMENT_TYPE
	}

	function emptyTarget(val) {
		return Array.isArray(val) ? [] : {}
	}

	function cloneUnlessOtherwiseSpecified(value, options) {
		return (options.clone !== false && options.isMergeableObject(value))
			? deepmerge(emptyTarget(value), value, options)
			: value
	}

	function defaultArrayMerge(target, source, options) {
		return target.concat(source).map(function(element) {
			return cloneUnlessOtherwiseSpecified(element, options)
		})
	}

	function getMergeFunction(key, options) {
		if (!options.customMerge) {
			return deepmerge
		}
		var customMerge = options.customMerge(key);
		return typeof customMerge === 'function' ? customMerge : deepmerge
	}

	function getEnumerableOwnPropertySymbols(target) {
		return Object.getOwnPropertySymbols
			? Object.getOwnPropertySymbols(target).filter(function(symbol) {
				return target.propertyIsEnumerable(symbol)
			})
			: []
	}

	function getKeys(target) {
		return Object.keys(target).concat(getEnumerableOwnPropertySymbols(target))
	}

	function propertyIsOnObject(object, property) {
		try {
			return property in object
		} catch(_) {
			return false
		}
	}

	// Protects from prototype poisoning and unexpected merging up the prototype chain.
	function propertyIsUnsafe(target, key) {
		return propertyIsOnObject(target, key) // Properties are safe to merge if they don't exist in the target yet,
			&& !(Object.hasOwnProperty.call(target, key) // unsafe if they exist up the prototype chain,
				&& Object.propertyIsEnumerable.call(target, key)) // and also unsafe if they're nonenumerable.
	}

	function mergeObject(target, source, options) {
		var destination = {};
		if (options.isMergeableObject(target)) {
			getKeys(target).forEach(function(key) {
				destination[key] = cloneUnlessOtherwiseSpecified(target[key], options);
			});
		}
		getKeys(source).forEach(function(key) {
			if (propertyIsUnsafe(target, key)) {
				return
			}

			if (propertyIsOnObject(target, key) && options.isMergeableObject(source[key])) {
				destination[key] = getMergeFunction(key, options)(target[key], source[key], options);
			} else {
				destination[key] = cloneUnlessOtherwiseSpecified(source[key], options);
			}
		});
		return destination
	}

	function deepmerge(target, source, options) {
		options = options || {};
		options.arrayMerge = options.arrayMerge || defaultArrayMerge;
		options.isMergeableObject = options.isMergeableObject || isMergeableObject;
		// cloneUnlessOtherwiseSpecified is added to `options` so that custom arrayMerge()
		// implementations can use it. The caller may not replace it.
		options.cloneUnlessOtherwiseSpecified = cloneUnlessOtherwiseSpecified;

		var sourceIsArray = Array.isArray(source);
		var targetIsArray = Array.isArray(target);
		var sourceAndTargetTypesMatch = sourceIsArray === targetIsArray;

		if (!sourceAndTargetTypesMatch) {
			return cloneUnlessOtherwiseSpecified(source, options)
		} else if (sourceIsArray) {
			return options.arrayMerge(target, source, options)
		} else {
			return mergeObject(target, source, options)
		}
	}

	deepmerge.all = function deepmergeAll(array, options) {
		if (!Array.isArray(array)) {
			throw new Error('first argument should be an array')
		}

		return array.reduce(function(prev, next) {
			return deepmerge(prev, next, options)
		}, {})
	};

	var deepmerge_1 = deepmerge;

	var cjs = deepmerge_1;

	var Lightning = window.lng;

	/**
	 * Simple module for localization of strings.
	 *
	 * How to use:
	 * 1. Create localization file with following JSON format:
	 * {
	 *   "en" :{
	 *     "how": "How do you want your egg today?",
	 *     "boiledEgg": "Boiled egg",
	 *     "softBoiledEgg": "Soft-boiled egg",
	 *     "choice": "How to choose the egg",
	 *     "buyQuestion": "I'd like to buy {0} eggs, {1} dollars each."
	 *   },
	 *
	 *   "it": {
	 *     "how": "Come vuoi il tuo uovo oggi?",
	 *     "boiledEgg": "Uovo sodo",
	 *     "softBoiledEgg": "Uovo alla coque",
	 *     "choice": "Come scegliere l'uovo",
	 *     "buyQuestion": "Mi piacerebbe comprare {0} uova, {1} dollari ciascuna."
	 *   }
	 * }
	 *
	 * 2. Use Locale's module load method, specifying path to your localization file and set chosen language, e.g.:
	 *    > Locale.load('static/locale/locale.json');
	 *    > Locale.setLanguage('en');
	 *
	 * 3. Use localization strings:
	 *    > console.log(Locale.tr.how);
	 *    How do you want your egg today?
	 *    > console.log(Locale.tr.boiledEgg);
	 *    Boiled egg
	 *
	 * 4. String formatting
	 *    > console.log(Locale.tr.buyQuestion.format(10, 0.5));
	 *    I'd like to buy 10 eggs, 0.5 dollars each.
	 */

	class Locale {
	  constructor() {
	    this.__enabled = false;
	  }

	  /**
	   * Loads translation object from external json file.
	   *
	   * @param {String} path Path to resource.
	   * @return {Promise}
	   */
	  async load(path) {
	    if (!this.__enabled) {
	      return
	    }

	    await fetch(path)
	      .then(resp => resp.json())
	      .then(resp => {
	        this.loadFromObject(resp);
	      });
	  }

	  /**
	   * Sets language used by module.
	   *
	   * @param {String} lang
	   */
	  setLanguage(lang) {
	    this.__enabled = true;
	    this.language = lang;
	  }

	  /**
	   * Returns reference to translation object for current language.
	   *
	   * @return {Object}
	   */
	  get tr() {
	    return this.__trObj[this.language]
	  }

	  /**
	   * Loads translation object from existing object (binds existing object).
	   *
	   * @param {Object} trObj
	   */
	  loadFromObject(trObj) {
	    this.__trObj = trObj;
	    for (const lang of Object.values(this.__trObj)) {
	      for (const str of Object.keys(lang)) {
	        lang[str] = new LocalizedString(lang[str]);
	      }
	    }
	  }
	}

	/**
	 * Extended string class used for localization.
	 */
	class LocalizedString extends String {
	  /**
	   * Returns formatted LocalizedString.
	   * Replaces each placeholder value (e.g. {0}, {1}) with corresponding argument.
	   *
	   * E.g.:
	   * > new LocalizedString('{0} and {1} and {0}').format('A', 'B');
	   * A and B and A
	   *
	   * @param  {...any} args List of arguments for placeholders.
	   */
	  format(...args) {
	    const sub = args.reduce((string, arg, index) => string.split(`{${index}}`).join(arg), this);
	    return new LocalizedString(sub)
	  }
	}

	var Locale$1 = new Locale();

	const settings = {};

	const initSettings = (appSettings, platformSettings) => {
	  settings['app'] = appSettings;
	  settings['platform'] = platformSettings;
	};

	// todo: support key for nested settings with dot notation? e.g. stage.useImageworker from 'app' settings
	var Settings = {
	  get(type, key) {
	    return settings[type] && settings[type][key]
	  },
	  has(type, key) {
	    return settings[type] && key in settings[type]
	  },
	};

	const prepLog = (type, args) => {
	  const colors = {
	    Info: 'green',
	    Debug: 'gray',
	    Warn: 'orange',
	    Error: 'red',
	  };

	  args = Array.from(args);
	  return [
	    '%c' + (args.length > 1 && typeof args[0] === 'string' ? args.shift() : type),
	    'background-color: ' + colors[type] + '; color: white; padding: 2px 4px; border-radius: 2px',
	    args,
	  ]
	};

	var Log = {
	  info() {
	    Settings.get('platform', 'log') && console.log.apply(console, prepLog('Info', arguments));
	  },
	  debug() {
	    Settings.get('platform', 'log') && console.debug.apply(console, prepLog('Debug', arguments));
	  },
	  error() {
	    Settings.get('platform', 'log') && console.error.apply(console, prepLog('Error', arguments));
	  },
	  warn() {
	    Settings.get('platform', 'log') && console.warn.apply(console, prepLog('Warn', arguments));
	  },
	};

	let sendMetric = (type, event, params) => {
	  Log.info('Sending metric', type, event, params);
	};

	const initMetrics = config => {
	  sendMetric = config.sendMetric;
	};

	// available metric per category
	const metrics = {
	  app: ['launch', 'loaded', 'ready', 'close'],
	  page: ['view', 'leave'],
	  user: ['click', 'input'],
	  media: [
	    'abort',
	    'canplay',
	    'ended',
	    'pause',
	    'play',
	    'suspend',
	    'volumechange',
	    'waiting',
	    'seeking',
	    'seeked',
	  ],
	};

	// error metric function (added to each category)
	const errorMetric = (type, message, code, visible, params = {}) => {
	  params = { params, ...{ message, code, visible } };
	  sendMetric(type, 'error', params);
	};

	const Metric = (type, events, options = {}) => {
	  return events.reduce(
	    (obj, event) => {
	      obj[event] = (name, params = {}) => {
	        params = { ...options, ...(name ? { name } : {}), ...params };
	        sendMetric(type, event, params);
	      };
	      return obj
	    },
	    {
	      error(message, code, params) {
	        errorMetric(type, message, code, params);
	      },
	      event(name, params) {
	        sendMetric(type, name, params);
	      },
	    }
	  )
	};

	const Metrics = types => {
	  return Object.keys(types).reduce(
	    (obj, type) => {
	      // media metric works a bit different!
	      // it's a function that accepts a url and returns an object with the available metrics
	      // url is automatically passed as a param in every metric
	      type === 'media'
	        ? (obj[type] = url => Metric(type, types[type], { url }))
	        : (obj[type] = Metric(type, types[type]));
	      return obj
	    },
	    { error: errorMetric, event: sendMetric }
	  )
	};

	var Metrics$1 = Metrics(metrics);

	class VersionLabel extends Lightning.Component {
	  static _template() {
	    return {
	      rect: true,
	      color: 0xbb0078ac,
	      h: 40,
	      w: 100,
	      x: w => w - 50,
	      y: h => h - 50,
	      mount: 1,
	      Text: {
	        w: w => w,
	        h: h => h,
	        y: 5,
	        text: {
	          fontSize: 22,
	          textAlign: 'center',
	        },
	      },
	    }
	  }

	  set version(version) {
	    this.tag('Text').text = `v${version}`;
	    this.tag('Text').loadTexture();
	    this.w = this.tag('Text').renderWidth + 40;
	  }
	}

	const defaultOptions = {
	  stage: { w: 1920, h: 1080, clearColor: 0x00000000, canvas2d: false },
	  debug: false,
	  defaultFontFace: 'RobotoRegular',
	  keys: {
	    8: 'Back',
	    13: 'Enter',
	    27: 'Menu',
	    37: 'Left',
	    38: 'Up',
	    39: 'Right',
	    40: 'Down',
	    174: 'ChannelDown',
	    175: 'ChannelUp',
	    178: 'Stop',
	    250: 'PlayPause',
	    191: 'Search', // Use "/" for keyboard
	    409: 'Search',
	  },
	};

	if (window.innerHeight === 720) {
	  defaultOptions.stage['w'] = 1280;
	  defaultOptions.stage['h'] = 720;
	  defaultOptions.stage['precision'] = 0.6666666667;
	}

	function Application(App, appData, platformSettings) {
	  Metrics$1.app.launch();
	  return class Application extends Lightning.Application {
	    constructor(options) {
	      const config = cjs(defaultOptions, options);
	      super(config);
	      this.config = config;
	    }

	    static _template() {
	      return {
	        w: 1920,
	        h: 1080,
	        rect: true,
	        color: 0x00000000,
	      }
	    }

	    _setup() {
	      Promise.all([
	        this.loadFonts((App.config && App.config.fonts) || (App.getFonts && App.getFonts()) || []),
	        Locale$1.load((App.config && App.config.locale) || (App.getLocale && App.getLocale())),
	      ])
	        .then(() => {
	          Metrics$1.app.loaded();
	          this.childList.a({
	            ref: 'App',
	            type: App,
	            appData,
	            forceZIndexContext: !!platformSettings.showVersion,
	          });

	          if (platformSettings.showVersion) {
	            this.childList.a({
	              ref: 'VersionLabel',
	              type: VersionLabel,
	              version: this.config.version,
	            });
	          }

	          super._setup();
	        })
	        .catch(console.error);
	    }

	    _handleBack() {
	      this.closeApp();
	    }

	    _handleExit() {
	      this.closeApp();
	    }

	    closeApp() {
	      Log.info('Closing App');
	      if (platformSettings.onClose && typeof platformSettings.onClose === 'function') {
	        Metrics$1.app.close();
	        platformSettings.onClose();
	      }
	      this.childList.remove(this.tag('App'));

	      // force texture garbage collect
	      this.stage.gc();
	    }

	    loadFonts(fonts) {
	      return new Promise((resolve, reject) => {
	        fonts
	          .map(({ family, url, descriptors }) => () => {
	            const fontFace = new FontFace(family, 'url(' + url + ')', descriptors || {});
	            document.fonts.add(fontFace);
	            return fontFace.load()
	          })
	          .reduce((promise, method) => {
	            return promise.then(() => method())
	          }, Promise.resolve(null))
	          .then(resolve)
	          .catch(reject);
	      })
	    }

	    _getFocused() {
	      return this.tag('App')
	    }
	  }
	}

	let basePath;
	let proxyUrl;

	const initUtils = config => {
	  if (config.path) {
	    basePath = ensureUrlWithProtocol(makeFullStaticPath(document.location.pathname, config.path));
	  }

	  if (config.proxyUrl) {
	    proxyUrl = ensureUrlWithProtocol(config.proxyUrl);
	  }
	};

	var Utils = {
	  asset(relPath) {
	    return basePath + '/' + relPath
	  },
	  proxyUrl(url, options = {}) {
	    return proxyUrl ? proxyUrl + '?' + makeQueryString(url, options) : url
	  },
	  makeQueryString() {
	    return makeQueryString(...arguments)
	  },
	  // since imageworkers don't work without protocol
	  ensureUrlWithProtocol() {
	    return ensureUrlWithProtocol(...arguments)
	  },
	};

	const ensureUrlWithProtocol = url => {
	  if (/^\/\//.test(url)) {
	    return window.location.protocol + url
	  }
	  if (!/^(?:https?:)/i.test(url)) {
	    return window.location.origin + url
	  }
	  return url
	};

	const makeFullStaticPath = (pathname = '/', path) => {
	  // if path is URL, we assume it's already the full static path, so we just return it
	  if (/^(?:https?:)?(?:\/\/)/.test(path)) {
	    return path
	  }
	  // cleanup the pathname
	  pathname = /(.*)\//.exec(pathname)[1];

	  // remove possible leading dot from path
	  path = path.charAt(0) === '.' ? path.substr(1) : path;
	  // ensure path has leading slash
	  path = path.charAt(0) !== '/' ? '/' + path : path;

	  return pathname + path
	};

	const makeQueryString = (url, options = {}, type = 'url') => {
	  // add operator as an option
	  options.operator = 'metrological'; // Todo: make this configurable (via url?)
	  // add type (= url or qr) as an option, with url as the value
	  options[type] = url;

	  return Object.keys(options)
	    .map(key => {
	      return encodeURIComponent(key) + '=' + encodeURIComponent('' + options[key])
	    })
	    .join('&')
	};

	class ScaledImageTexture extends Lightning.textures.ImageTexture {
	  constructor(stage) {
	    super(stage);
	    this._scalingOptions = undefined;
	  }

	  set options(options) {
	    this.resizeMode = this._scalingOptions = options;
	  }

	  _getLookupId() {
	    return `${this._src}-${this._scalingOptions.type}-${this._scalingOptions.w}-${this._scalingOptions.h}`
	  }

	  getNonDefaults() {
	    const obj = super.getNonDefaults();
	    if (this._src) {
	      obj.src = this._src;
	    }
	    return obj
	  }
	}

	var Img = (imageUrl, options) => {
	  const imageServerUrl = Settings.get('platform', 'imageServerUrl');

	  // make and return ScaledImageTexture
	  const make = options => {
	    // local asset, wrap it in Utils.asset()
	    if (!/^(?:https?:)?\/\//i.test(imageUrl)) {
	      imageUrl = Utils.asset(imageUrl);
	    }

	    // only pass to image server if imageServerUrl is configured
	    // and if the asset isn't local to the app (i.e. has same origin)
	    if (imageServerUrl && imageUrl.indexOf(window.location.origin) === -1) {
	      imageUrl = Utils.ensureUrlWithProtocol(
	        imageServerUrl + '?' + Utils.makeQueryString(imageUrl, options)
	      );
	    } else {
	      // Lightning will handle the resizing and has only 2 flavours (cover and contain)
	      if (options.type === 'crop') options.type = 'cover';
	      else options.type = 'contain';
	    }

	    return {
	      type: ScaledImageTexture,
	      src: imageUrl,
	      options: options,
	    }
	  };

	  // merge options with default
	  const setOptions = options => {
	    return {
	      ...{
	        type: 'contain',
	        w: 0,
	        h: 0,
	      },
	      ...options,
	    }
	  };

	  // if options are passed, return scaled image right away
	  if (options) {
	    return make(setOptions(options))
	  }

	  // otherwise return 'chained' functions
	  return {
	    // official api
	    exact: (w, h) => make(setOptions({ type: 'exact', w, h })),
	    landscape: w => make(setOptions({ type: 'landscape', w })),
	    // fixme: remove deprecated stuff in future release of SDK
	    portrait: (w_but_actually_h, deprecated_h) => {
	      if (typeof deprecated_h !== 'undefined' && deprecated_h !== null) {
	        console.warn(
	          "The signature of the 'portrait()'-method has changed, soon it will only accept the height as a single argument (instead of 'portrait(width, height)')"
	        );
	        w_but_actually_h = deprecated_h;
	      }
	      return make(setOptions({ type: 'portrait', h: w_but_actually_h }))
	    },
	    cover: (w, h) => make(setOptions({ type: 'cover', w, h })),
	    contain: (w, h) => make(setOptions({ type: 'contain', w, h })),
	    original: () => make(setOptions({ type: 'contain' })),

	    // todo: add positioning - i.e. top, bottom, center, left etc.

	    // FIXME: remove deprecated api (summer of 2020)
	    crop: (w, h) => {
	      console.warn(
	        "The 'crop()'-method is deprecated and will be removed. Please use 'cover()' instead"
	      );
	      return make(setOptions({ type: 'cover', w, h }))
	    },
	    fit: (w, h) => {
	      console.warn(
	        "The 'fit()'-method is deprecated and will be removed. Please use 'exact()' instead"
	      );
	      return make(setOptions({ type: 'exact', w, h }))
	    },
	    parent: (w, h) => {
	      console.warn(
	        "The 'parent()'-method is deprecated and will be removed. Please use 'exact()' instead"
	      );
	      return make(setOptions({ type: 'exact', w, h }))
	    },
	    height: (w, h) => {
	      console.warn(
	        "The 'height()'-method is deprecated and will be removed. Please use 'portrait()' instead"
	      );
	      return make(setOptions({ type: 'portrait', w, h }))
	    },
	    width: (w, h) => {
	      console.warn(
	        "The 'width()'-method is deprecated and will be removed. Please use 'landscape()' instead"
	      );
	      return make(setOptions({ type: 'landscape', w, h }))
	    },
	    auto: (w, h) => {
	      console.warn(
	        "The 'auto()'-method is deprecated and will be removed. Please use 'cover()' instead"
	      );
	      return make(setOptions({ type: 'contain', w, h }))
	    },
	  }
	};

	let getInfo = () => Promise.resolve();
	let setInfo = () => Promise.resolve();

	const initProfile = config => {
	  getInfo = config.getInfo;
	  setInfo = config.setInfo;
	};

		   /*
	const events = [
	  'timeupdate',
	  'error',
	  'ended',
	  'loadeddata',
	  'canplay',
	  'play',
	  'playing',
	  'pause',
	  'loadstart',
	  'seeking',
	  'seeked',
	  'encrypted',
	];
			*/
		   
	let mediaUrl = url => url;

	const initMediaPlayer = config => {
	  if (config.mediaUrl) {
	    mediaUrl = config.mediaUrl;
	  }
	};

	class Mediaplayer extends Lightning.Component {
	  _construct() {
		console.log( "entering Mediaplayer::_construct" );
	    this._metrics = null;
	  }

	  static _template() {
	    return {
	      Video: {
		   VideoWrap: {
	          VideoTexture: {
	            visible: false,
	            pivot: 0.5,
	            texture: { type: Lightning.textures.StaticTexture, options: {} },
	          },
	        },
	      },
	    }
	  }

	  set skipRenderToTexture(v) {
		   // STUB
	  }

	  get textureMode() {
		   // STUB
		return false;
	  }

	  get videoView() {
		console.log( "entering videoView" );
	    return this.tag('Video')
	  }

	  _init() {
		console.log( "entering _init()" );
	    const videoEls = document.getElementsByTagName('video');
	    if (videoEls && videoEls.length > 0)
		{ // recycle existing videotag
		   this.videoEl = videoEls[0];
		}
		else
		{ // dummy video for hole punching
		   this.videoEl = document.createElement('video');
		   this.videoEl.style.position = 'absolute';
		   this.videoEl.setAttribute('src','dummy.mp4');
		   document.body.appendChild(this.videoEl);
		}
		if( !this._RdkMediaPlayer )
		{
		   console.log( "Creating AAMP Media Player" );
		   try
		   {
			this._RdkMediaPlayer = new AAMPMediaPlayer();
		   }
		   catch( e )
		   {
		   console.log( "RDK Media Player not available in this environment" );
		   }
		}
	  }
		   
	  _registerListeners() {
		console.log( "entering _registerListeners" );
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.addEventListener("playbackStateChanged", this._playbackStateChanged );
		   this._RdkMediaPlayer.addEventListener("playbackProgressUpdate", this._playbackProgressUpdate);
		   this._RdkMediaPlayer.addEventListener("durationChanged", this._durationChanged);
		   this._RdkMediaPlayer.addEventListener("seeked", this._seeked);
		}
	  }

	  _deregisterListeners() {
		console.log( "entering _deregisterListeners" );
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.removeEventListener("playbackStateChanged", this._playbackStateChanged );
		   this._RdkMediaPlayer.removeEventListener("playbackProgressUpdate", this._playbackProgressUpdate);
		   this._RdkMediaPlayer.removeEventListener("durationChanged", this._durationChanged);
		   this._RdkMediaPlayer.removeEventListener("seeked", this._seeked);
		}
	  }

	  _attach() {
		console.log( "entering _attach" );
	    this._registerListeners();
	  }

	  _detach() {
		console.log( "entering _detach" );
	    this._deregisterListeners();
	    this.close();
	  }

	  _createVideoTexture() { // needed?
		   console.log( "_createVideoTexture" );
	  }

	  _startUpdatingVideoTexture() {
		  console.log( "_startUpdatingVideoTexture" );
	  }

	  _stopUpdatingVideoTexture() {
		  console.log( "_stopUpdatingVideoTexture" );
	  }

	  updateSettings(settings = {}) {
		console.log( "updateSettings" );
	    // The Component that 'consumes' the media player.
	    this._consumer = settings.consumer;
	    this._setHide(settings.hide);
	    this._setVideoArea(settings.videoPos);
	  }

	  _setHide(hide) {
		console.log("_setHide " + hide );
	  }

	open(url, settings = { hide: false, videoPosition: null }) {
		console.log( "entering open " + url );
	    // prep the media url to play depending on platform (mediaPlayerplugin)
		url = mediaUrl(url);
		var delim = url.indexOf("?");
		if( delim>=0 )
		{
			if( url.indexOf("/",delim)>=0 )
			{ // temporary workaround for DELIA-42658
				url = url.substr(0,delim);
			}
		}
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.load(url);
		}
	    this._metrics = Metrics$1.media(url);
	    Log.info('Playing stream', url);
	    this._setVideoArea(settings.videoPosition);
		this.videoEl.style.display = 'block';
		this._setHide(settings.hide);
		this._isPlaying = true;
		   
		   this._fireConsumer('$mediaplayerPlay',null);
//		   setTimeout(this._delayedFunction, 3000);
	  }

	  close() {
		   console.log("entering close" );
		   if( this._RdkMediaPlayer )
		   {
				this._RdkMediaPlayer.stop();
				// this._RdkMediaPlayer.release();
				// this._RdkMediaPlayer = null;
		   }
		   this.videoEl.style.display = 'none';
		   this._isPlaying = false;
	  }

	  playPause() {
		console.log( "entering playPause" );
	    if (this.isPlaying()) {
	      this.doPause();
	    } else {
	      this.doPlay();
	    }
	  }

	  get muted() {
		console.log( "entering get muted");
		return false; // TODO
	  }

	  set muted(v) {
		console.log("entering set muted");
	  }

	  get loop() {
		console.log("entering get loop");
		return false; // TODO
	  }

	  set loop(v) {
		// TODO
	  }

	  isPlaying() {
		console.log( "isPlaying" );
		return this._isPlaying;
		//return this._RdkMediaPlayer && this._RdkMediaPlayer.getCurrentState()==8;
	  }

	  doPlay() {
		console.log( "entering doPlay" );
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.play();
		}
	  }

	  doPause() {
		console.log( "doPause" );
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.pause();
		}
	  }

	  reload() {
		   // STUB
	  }

	  getPosition() {
		if( this._RdkMediaPlayer )
		{
		   return this._RdkMediaPlayer.getCurrentPosition();
		}
		return 0;
	  }

	  setPosition(pos) {
		console.log("entering setPosition" );
		if( this._RdkMediaPlayer )
		{
		   this._RdkMediaPlayer.seek(pos);
		}
	  }

	  getDuration() {
		if( this._RdkMediaPlayer )
		{
		   return this._RdkMediaPlayer.getDurationSec();
		}
		return 0;
	  }

	  seek(time, absolute = false) {
		if( this._RdkMediaPlayer )
		{
			if( absolute )
			{
				this._RdkMediaPlayer.seek(time);
		    }
			else
			{
				this._RdkMediaPlayer.seek(time+this._RdkMediaPlayer.getDurationSec());
		    }
		}
	  }

	  get videoTextureView() {
		return 0; // STUB
	  }

	  get videoTexture() {
		return 0; // STUB
	  }

	  _setVideoArea(videoPos) {
		   if( !videoPos ) videoPos = [0, 0, 1920, 1080];
		   console.log( "entering _setVideoArea " + videoPos );
		   if( this._RdkMediaPlayer )
		   {
				var x = Math.round(videoPos[0]);
				var y = Math.round(videoPos[1]);
				var w = Math.round(videoPos[2]);
				var h = Math.round(videoPos[3]);
				if( this._RdkMediaPlayer )
				{
					this._RdkMediaPlayer.setVideoRect( x, y, w, h );
				}
				this.videoEl.style.left = x + 'px';
				this.videoEl.style.top = y + 'px';
				this.videoEl.style.width = w + 'px';
				this.videoEl.style.height = h + 'px';
		   }
	  }

	  _fireConsumer(event, args) {
	    if (this._consumer) {
	      this._consumer.fire(event, args);
	    }
	  }

_playbackStateChanged(e)
{
	console.log("_playbackStateChanged " + JSON.stringify(e) );
	switch( event.state )
	{
		case 0: // eSTATE_IDLE
		    break;
		case 1: // eSTATE_INITIALIZING
		   break;
		case 2: // eSTATE_INITIALIZED
		   break;
		case 6: // eSTATE_PAUSED
		   this._fireConsumer('$mediaplayerPause', e);
		   break;
		   
		case 7: // eSTATE_SEEKING
		   this._fireConsumer('$mediaplayerSeeking', {
			 currentTime: this._RdkMediaPlayer.getCurrentPosition(),
			 duration: this._RdkMediaPlayer.getDurationSec()
		   });
		   break;
		   
		case 8: // eSTATE_PLAYING
//		   this._fireConsumer('$mediaplayerPlay', e);
		   break;
		
		case 9: // eSTATE_STOPPING
		   break;
		
		case 10: // eSTATE_STOPPED
		   this._fireConsumer('$mediaplayerStop', {});
		   break;
		   
		case 11: // eSTATE_COMPLETE
		   this._fireConsumer('$mediaplayerEnded', e);
		   break;
		   
		case 12: // eSTATE_ERROR
		   this._fireConsumer('$mediaplayerError',e);
		   break;
		   
		case 13: // eSTATE_RELEASED
		   break;
		   // this._fireConsumer('$mediaplayerStart', args);
		   // this._fireConsumer('$mediaplayerLoad', args);
		   // this._fireConsumer('$mediaplayerLoadedData', e);
	}
}
		   
_playbackProgressUpdate(e)
{
//	console.log("_playbackProgressUpdate " + JSON.stringify(e) );
	this._fireConsumer('$mediaplayerProgress', {
			currentTime: this._RdkMediaPlayer.getCurrentPosition(),
			duration: this._RdkMediaPlayer.getDurationSec()
	});
}
_durationChanged(e)
{
	console.log("UVE event: duration changed" + JSON.stringify(e) );
	this._fireConsumer('$mediaplayerDurationChange', e);
}
_seeked(e)
{
	console.log("UVE event: seeked" + JSON.stringify(e) );
	this._fireConsumer('$mediaplayerSeeked', {
		currentTime: this._RdkMediaPlayer.getCurrentPosition(), //this.videoEl.currentTime,
		duration: this._RdkMediaPlayer.getDurationSec()
	});
}

	  static _states() {
	    return [];
	  }
	}

	// LocalStorage and Cookie storage library
	class localCookie {
		constructor(options) {
			options = options || {};
			this.forceCookies = options.forceCookies || false;

			if (this._checkIfLocalStorageWorks() === true && options.forceCookies !== true)
				return {
					getItem : this._getItemLocalStorage,
					setItem : this._setItemLocalStorage,
					removeItem : this._removeItemLocalStorage,
					clear : this._clearLocalStorage,
					keys : this._getLocalStorageKeys,
				}
			else
				return {
					getItem : this._getItemCookie,
					setItem : this._setItemCookie,
					removeItem : this._removeItemCookie,
					clear : this._clearCookies,
					keys : this._getCookieKeys,
				}
		}

		// Checks if local storage is available and works
		_checkIfLocalStorageWorks() {
			if (typeof localStorage !== 'undefined') {
				try {
					localStorage.setItem('feature_test', 'yes');
					if (localStorage.getItem('feature_test') === 'yes') {
						localStorage.removeItem('feature_test');
						// localStorage is enabled
						return true
					} else {
						// localStorage is disabled
						return false
					}
				} catch(e) {
					// localStorage is disabled
					return false
				}
			} else {
				// localStorage is not available
				return false
			}
		}

		/*
		 * Local Storage APIs
		 */
		 _getItemLocalStorage(key) {
		 	return window.localStorage.getItem(key)
		 }

		 _setItemLocalStorage(key, value) {
		 	return window.localStorage.setItem(key, value)
		 }

		 _removeItemLocalStorage(key) {
		 	return window.localStorage.removeItem(key)
		 }

		 _clearLocalStorage() {
		 	return window.localStorage.clear()
		 }

		 _getLocalStorageKeys() {
			return Object.keys(window.localStorage)
		 }

		 /*
		  * Cookie APIs
		  */
		_getItemCookie(key) {
			function escape(s) { return s.replace(/([.*+?\^${}()|\[\]\/\\])/g, '\\$1'); }		var match = document.cookie.match(RegExp('(?:^|;\\s*)' + escape(key) + '=([^;]*)'));

			//if value in key of the entire cookie is empty, its deleted
			if (match && match[1] === '')
				match[1] = null;

			return match ? match[1] : null;
		}

		_setItemCookie(key, value) {
			document.cookie = `${key}=${value}`;
			return //return undefined just like localStorage
		}

		_removeItemCookie(key) {
			// to delete an k/v set the key to empty
			document.cookie = `${key}=;Max-Age=-99999999;`;
			return //return undefined just like localStorage
		}

		_clearCookies() {
			// empty all key/values in cookies
			// What about cookies we got from the server? ¯\_(ツ)_/¯
			// They should have the HttpOnly flag, so we shouldn't be able to delete them
			document.cookie.split(";").forEach( (c) => {
				document.cookie = c.replace(/^ +/, "").replace(/=.*/, "=;expires=Max-Age=-99999999");
			});

			return //return undefined
		}

		_getCookieKeys() {
			return document.cookie.split(';').map(cookie => {
				const parts = cookie.split('=');
				return parts[1] !== '' ? parts[0].trim() : false
			}).filter(key => key)
		}
	}

	let namespace;
	let lc;

	const initStorage = () => {
	  namespace = Settings.get('platform', 'appId');
	  // todo: pass options (for example to force the use of cookies)
	  lc = new localCookie();
	};

	const namespacedKey = key => (namespace ? [namespace, key].join('.') : key);

	var Storage = {
	  get(key) {
	    try {
	      return JSON.parse(lc.getItem(namespacedKey(key)))
	    } catch (e) {
	      return null
	    }
	  },
	  set(key, value) {
	    try {
	      lc.setItem(namespacedKey(key), JSON.stringify(value));
	      return true
	    } catch (e) {
	      return false
	    }
	  },
	  remove(key) {
	    lc.removeItem(namespacedKey(key));
	  },
	  clear() {
	    if (namespace) {
	      lc.keys().forEach(key => {
	        // remove the item if in the namespace
	        key.indexOf(namespace + '.') === 0 ? lc.removeItem(key) : null;
	      });
	    } else {
	      lc.clear();
	    }
	  },
	};

	var Launch = (App, appSettings, platformSettings, appData) => {
	  initSettings(appSettings, platformSettings);

	  initUtils(platformSettings);
	  initStorage();

	  // Initialize plugins
	  if (platformSettings.plugins) {
	    platformSettings.plugins.profile && initProfile(platformSettings.plugins.profile);
	    platformSettings.plugins.metrics && initMetrics(platformSettings.plugins.metrics);
	    platformSettings.plugins.mediaPlayer && initMediaPlayer(platformSettings.plugins.mediaPlayer);
	  }

	  const app = Application(App, appData, platformSettings);
	  return new app(appSettings)
	};

	class SpinningLogo extends Lightning.Component {

	    static _template() {
	        return {
	            src: Utils.asset('images/splash-spinner.png')
	        };
	    }

	    _setup() {
	        this._loader = this.animation({duration: 1, repeat: -1, stopMethod: 'immediate',
	            actions: [{ p: 'rotation', v: { 0: {v: 0, sm:0}, 1:{v: 2 * Math.PI, sm:0}}}]
	        });
	    }

	    _active() {
	        this._loader.start();
	    }

	    _inactive() {
	        this._loader.stop();
	    }

	    _handleBack() {
	        return false;
	    }

	    _handleKey() {
	        //block all the key events other than BACK
	    }
	}

	var Config = {
	    COLORS: {
	        BACKGROUND: 0xfff3f7f5
	    },
	    LIVE_PROGRAMS_POLL_FREQUENCY: 60 * 1000, // 1 minute
	    SELECTED_LANGUAGE : 'SelectedLanguage' //Key to save the selected language information on the local storage.
	};

	class Splash extends Lightning.Component {

	    static _template() {
	        return {
	            Background: { w: 1920, h: 1080, rect: true, color: Config.COLORS.BACKGROUND},
	            Wrapper: {
	                x: 960, y: 540, mount: 0.5 , flex: {direction: 'column', alignItems: 'center'} ,
	                Logo: {flexItem: {}, zIndex: 2, src: Utils.asset('images/france-24-logo.png')},
	                Shadow: {
	                    zIndex: 1,
	                    color: 0x26000000,
	                    texture: Lightning.Tools.getShadowRect(270, 8, 0, 6, 0),
	                    alpha: 0
	                },
	                Spinner: {flexItem: {marginTop: 60}, zIndex: 2, type: SpinningLogo}
	            }
	        };
	    }

	    _init() {
	        this.tag('Logo').on('txLoaded', () => {
	            this.tag('Shadow').patch({
	                alpha: 1
	            });
	            this.fireAncestors('$onReady');
	        });
	    }

	    _getFocused() {
	        return this.tag("Spinner");
	    }

	}

	class MenuItem extends Lightning.Component {

	    static _template() {
	        return {
	            FlexItem: {
	                flex: {direction: 'column', alignItems: 'stretch'},
	                Title: {mountX:0.5, text: {fontFace: 'OpenSans', fontSize: 40, lineHeight:55, fontStyle:'normal', textColor: 0xff101010}},
	                Line: {mountX:0.5, alpha: 1, y:-20, h: 4, color: 0xff101010 , rect:true}
	            }
	        }
	    }

	    set title(title) {
	        this.patch({
	            FlexItem: {
	                Title: {text: {text: title.toUpperCase()}},
	            }
	        });
	    }

	    set selection(t) {
	        this._selected = t;
	        this.tag('FlexItem').patch({
	            Title: {text: {textColor: 0xff101010, fontStyle: t ? '600' : 'normal'}},
	            Line: {color: 0xff101010, alpha: t ? 1 : 0}
	        });
	    }

	    _focus() {
	        this.patch({
	            FlexItem: {
	                Title: {text: {textColor: 0xff00a7e3, fontStyle: '600'}},
	                Line: {color: 0xff00a7e3, alpha: 1}
	            }
	        });
	    }

	    _unfocus() {
	        this.patch({
	            FlexItem: {
	                Title: { text: {textColor: 0xff101010, fontStyle: this._selected ? '600' : 'normal'}},
	                Line: {alpha : this._selected ? 1 : 0, color: 0xff101010}
	            }
	        });
	    }

	}

	class Menu extends Lightning.Component {

	    static _template() {
	        return {
	            x: 270, y: 57,
	            Menu1: {alpha:0},
	            Menu:{
	                w:w=>w,
	                flex:{direction:'row'}
	            },
	        }
	    }

	   _init() {
	        this._setItems();
	        this._focusIdx = 0;
	        this._selectedIdx = 0;
	   }

	   _setItems(){
	       this.tag("Menu1").children = Menu.ITEMS.map(item => {
	           return {
	               type: MenuItem,
	               title: Locale$1.tr[item.title],
	               selection: item.selection,
	               x: item.x
	           }
	       });

	       this.tag("Menu").children = Menu.ITEMS.map(item => {
	           return {
	               flexItem:{marginRight:70},
	               type: MenuItem1,
	               title: Locale$1.tr[item.title],
	               selection: item.selection,
	           }
	       });
	   }
	    reset(){
	        this._setItems();
	        this._focusIdx = 0;
	        this._selectedIdx = 0;

	        this._handleEnter();
	    }

	   _getFocused() {
	       return this.tag("Menu").children[this._focusIdx];
	   }

	   _handleRight() {
	        if(this._focusIdx < Menu.ITEMS.length - 1) {
	            this._focusIdx++;
	        }
	   }

	   _handleEnter() {
	        this._selectedIdx = this._focusIdx;
	        this.tag("Menu").children.forEach((item, index) => {
	           item.selection = (this._selectedIdx === index);
	        });
	       this.fireAncestors("$select", Menu.ITEMS[this._focusIdx].title);
	   }

	   _handleLeft() {
	        if(this._focusIdx > 0) {
	            this._focusIdx--;
	        }
	   }

	   _handleDown() {
	       /*Reset the focused to selected. So that the next time when the focus is gained by Menu,
	       then the selected will be highlighted.
	        */
	        this._focusIdx = this._selectedIdx;

	        return false;
	   }

	}

	Menu.ITEMS = [
	    {title: "topStories", selection: true, x: 0},
	    {title: "mostViewed", selection: false, x: 332},
	    {title: "shows", selection: false, x: 610},
	    {title: "languages", selection: false, x: 872},
	    {title: "about", selection: false, x: 1132}
	];



	class MenuItem1 extends Lightning.Component {

	    static _template() {
	        return {
	            h:50,
	            FlexItem: {
	                Title: {text: {fontFace: 'OpenSans', fontSize: 40, lineHeight:55, fontStyle:'normal', textColor: 0xff101010}},
	                Line: {alpha: 1, y:55, h: 4, color: 0xff101010 , rect:true}
	            }
	        }
	    }

	    _init(){
	        this._initialized = false;
	        this.tag('Title').on('txLoaded',()=>{
	            if(!this._initialized) {
	                this._initialized = true;
	                this.tag('Line').setSmooth('w', this.tag('Title').finalW, {duration: 0.3});
	                this.w = this.tag('Title').finalW;
	            }
	        });
	    }
	    set title(title) {
	        this.patch({
	            FlexItem: {
	                Title: {text: {text: title.toUpperCase()}},
	            }
	        });
	    }

	    set selection(t) {
	        this._selected = t;
	        this.tag('FlexItem').patch({
	            Title: {text: {textColor: 0xff101010, fontStyle: t ? '600' : 'normal'}},
	            Line: {color: 0xff101010, alpha: t ? 1 : 0}
	        });
	    }

	    _focus() {
	        this.patch({
	            FlexItem: {
	                Title: {text: {textColor: 0xff00a7e3, fontStyle: '600'}},
	                Line: {color: 0xff00a7e3, alpha: 1}
	            }
	        });
	    }

	    _unfocus() {
	        this.patch({
	            FlexItem: {
	                Title: { text: {textColor: 0xff101010, fontStyle: this._selected ? '600' : 'normal'}},
	                Line: {alpha : this._selected ? 1 : 0, color: 0xff101010}
	            }
	        });
	    }

	}

	class Header extends Lightning.Component {

	    static _template() {
	        return {
	            w: 1920, h: 140, rect: true, color: 0xffffffff,
	            Line: {
	                x: 0, y: 140, w: 1920, h: 2, rect: true, color: 0xffa9adac
	            },
	            Image: {
	                x: 100, y: 50, zIndex: 2, src: Utils.asset('images/france-24-logo-small.png')
	            },
	            Shadow: {
	                x: 70, y: 120,
	                zIndex: 1,
	                color: 0x35000000,
	                texture: Lightning.Tools.getShadowRect(100, 3, 0, 4,30),
	            },
	            Menu: {
	                type: Menu
	            }
	        }
	    }

	    _getFocused() {
	        return this.tag("Menu");
	    }

	    reset(){
	        this.tag('Menu').reset();
	    }
	}

	class List extends Lightning.Component {
	    static _template() {
	        return {
	            Title: {
	                text: {fontSize: 44, fontFace: 'OpenSans', fontStyle: '600'}, color: 0xff606060, alpha: 0
	            },
	            List: {
	                type: Lightning.components.ListComponent,
	                w: 1920,
	                x: -85,
	                scrollTransition: {duration: 0.2},
	                invertDirection: false,
	                roll: true,
	                viewportScrollOffset: 0.5,
	                itemScrollOffset: 0.5,
	                rollMin: 90,
	                rollMax: 90
	            }
	        };
	    }

	    set title(title) {
	        if(title) {
	            this.patch({
	                Title: {text: {text: title}, smooth: {alpha: 1}},
	                List: {smooth: {y: 80}}
	            });
	        }
	    }

	    set itemSize(size) {
	        this.patch({
	            List: {itemSize: size}

	        });
	    }

	    resetFocus() {
	        this.tag('List').setIndex(0);
	    }

	    get items() {
	        return this._items;
	    }

	    get active() {
	        return this.tag('List').getElement(this.index);
	    }

	    get index() {
	        return this.tag('List').realIndex;
	    }

	    set items(items) {
	        this._items = items;
	        this.tag('List').items = this._items;
	    }

	    set storeFocus(v) {
	        this._storeFocus = v;
	    }

	    _handleLeft() {
	       this.tag('List').setPrevious();
	    }

	    _unfocus() {
	        if (!this._storeFocus) {
	            this.tag('List').setIndex(0);
	        }
	    }

	    _handleRight() {
	        this.tag('List').setNext();
	    }

	    _getFocused() {
	        return this.active;
	    }
	}

	class TransparentShader extends  Lightning.shaders.WebGLDefaultShader {
	    beforeUsage() {
	        this.gl.disable(this.gl.BLEND);
	    }

	    afterUsage() {
	        this.gl.enable(this.gl.BLEND);
	    }
	}

	class PrimaryItem extends Lightning.Component {
	    static _template() {
	        return {
	            w: 720, h: 414,
	            TransparentRect: {rect: true, color: 0x00000000, w: 700, h: 394, alpha : 0},
	            PrimaryImage: {w: 700, h: 394},
	            DefaultImage: {w: 700, h: 394, src: Utils.asset('images/default-image.png')},
	            FocusIndicator: {w: 170, h: 170, x: 360, y: 207, mount: 0.5, alpha: 0, src: Utils.asset('images/play-big.png')},
	            Border: {
	                alpha: 0,
	                Border: {type: Lightning.components.BorderComponent, borderWidth: 10, colorBorder: 0xff00a7e3, w: 700 , h: 394}
	            },
	        };

	    }

	    _init() {
	        this.tag('PrimaryImage').on('txLoaded', () => {
	            this._imageLoaded = true;
	            this.patch({
	                DefaultImage: {smooth: {alpha: 0}},
	            });
	        });
	    }

	    set image(image) {
	        this.patch({
	            PrimaryImage: {
	                src: image
	            }
	        });
	    }

	    set imageUrl(image) {
	        this.patch({
	            PrimaryImage: {
	                texture: Img(image).cover(700, 394),
	                alpha: 1
	            }
	        });
	    }

	    set hidePlayOnFocus(v) {
	        this._hidePlayOnFocus = v;
	    }

	    set transparent(v) {
	        this.patch({
	            TransparentRect: {
	                alpha: v ? 1 : 0, shader: v ? {type: TransparentShader} : null
	            },
	            PrimaryImage: {
	                alpha: v ? 0 : 1
	            },
	            DefaultImage: {
	                alpha: v ? 0 : this._imageLoaded ? 0 : 1
	            }
	        });
	    }

	    _focus() {
	        this.patch({
	            smooth: {scale: 1.02},
	            Border: {
	                smooth: {alpha: 1}
	            },
	            FocusIndicator: {
	                smooth: {alpha: this._hidePlayOnFocus ? 0 : 1}
	            }
	        });
	    }

	    _unfocus() {
	        this.patch({
	            smooth: {scale: 1},
	            Border: {
	                smooth: {alpha: 0}
	            },
	            FocusIndicator: {
	                smooth: {alpha: 0}
	            }
	        });
	    }
	}

	class ImageItem extends Lightning.Component {
	    static _template() {
	        return {
	            w: 370, h: 205,
	            Shadow: {
	                x: -28, y: 174,
	                color: 0x35000000,
	                texture: Lightning.Tools.getShadowRect(368, 6, 0, 3, 30),
	            },
	            Image: {w: 370, h: 205},
	            DefaultImage: {w: 370, h:205, src: Utils.asset('images/default-image.png')},
	            Border: {
	                alpha: 0,
	                Border: {type: Lightning.components.BorderComponent, borderWidth: 6, colorBorder: 0xff00a7e3, w: 370 , h: 205}
	            },
	        };

	    }

	    _init() {
	        this.tag('Image').on('txLoaded', () => {
	            this.patch({
	                DefaultImage: {smooth: {alpha: 0}},
	            });
	        });
	    }

	    set item(v) {
	        this._item = v;
	        this.patch({
	            Image: {
	                texture: Img(v.getPicture(388)).cover(370, 205)
	            }
	        });

	    }

	    get item() {
	        return this._item;
	    }

	    _focus() {
	        this.patch({
	            smooth: {scale: 1.1},
	            Border: {
	                smooth: {alpha: 1}
	            },
	            Shadow : {
	                smooth: {alpha: 0}
	            }
	        });
	    }

	    _unfocus() {
	        this.patch({
	            smooth: {scale: 1},
	            Border: {
	                smooth: {alpha: 0}
	            },
	            Shadow : {
	                smooth: {alpha: 1}
	            }
	        });
	    }
	}

	const formatTime = (millis) => {
	    let monthNames = [
	        "January", "February", "March",
	        "April", "May", "June", "July",
	        "August", "September", "October",
	        "November", "December"
	    ];

	    let date = new Date(millis);
	    let onlyDate = date.getDate();
	    onlyDate = onlyDate < 10 ? `0${onlyDate}` : onlyDate;

	    return `${monthNames[date.getMonth()]} ${onlyDate}, ${date.getFullYear()}`;
	};

	class VideoItem extends Lightning.Component {
	    static _template() {
	        return {
	            w: 370, h: 365,
	            Shadow: {
	                x: -28, y: 174,
	                color: 0x35000000,
	                texture: Lightning.Tools.getShadowRect(368, 6, 0, 3, 30),
	            },
	            Image: {w: 370, h: 205},
	            DefaultImage: {w: 370, h:205, src: Utils.asset('images/default-image.png')},
	            Border: {
	                alpha: 0, Border: {type: Lightning.components.BorderComponent, borderWidth: 6, colorBorder: 0xff00a7e3, w: 370 , h: 205}
	            },
	            Title: {
	                y: 228, text: {fontFace: 'OpenSans', fontSize: 22, fontStyle: 'bold', maxLines: 3, wordWrapWidth: 370, lineHeight: 32}, color: 0xff101010
	            },
	            Date: {
	                y: 333, text: {fontFace: 'OpenSans', fontSize: 22, fontStyle: '600'}, color: 0xff909090
	            }
	        };

	    }

	    _init() {
	        this.tag('Image').on('txLoaded', () => {
	            this.patch({
	                DefaultImage: {smooth: {alpha: 0}},
	            });
	        });
	    }

	    set item(v) {
	        this._item = v;
	        this.patch({
	            Image: {
	                texture: Img(v.getPicture(388)).cover(370,205)
	            },
	            Title: {text: {text: v.description || ''}},
	            Date: {text: {text: v.createdTime ? formatTime(v.createdTime) : ''}}
	        });
	    }

	    get item() {
	        return this._item;
	    }

	    _focus() {
	        this.patch({
	            Title: {color: 0xff00a7e3},
	            Border: {
	                smooth: {alpha: 1}
	            },
	            Shadow : {
	                smooth: {alpha: 0}
	            },
	            smooth: {scale: 1.1},
	        });
	    }

	    _unfocus() {
	        this.patch({
	            Title: {color: 0xff101010},
	            Border: {
	                smooth: {alpha: 0}
	            },
	            Shadow : {
	                smooth: {alpha: 1}
	            },
	            smooth: {scale: 1}
	        });
	    }
	}

	class Alert extends Lightning.Component{
	    static _template(){
	        return {
	            x:960, y:540, mount:0.5,
	            w:760, h:360, zIndex:2,
	            Shadow: {
	                x:-17, y: -17,color: 0xB2000000,
	                texture:Lightning.Tools.getShadowRect(760,360, 0, 20, 20)
	            },
	            Background: {
	                w:w=>w, h:h=>h, color: 0xfff3f7f4, rect: true
	            },
	            Wrapper:{
	                x:30, w:w=>w,  y:42,
	                Title: {
	                    w:w=>w,
	                    Text: {
	                        x:(w=>(w-60)/2), mountX:0.5,
	                        text: {
	                            fontFace: 'OpenSans',
	                            textAlign: 'center',
	                            fontSize: 32,
	                            lineHeight: 43,
	                            fontStyle: '600',
	                            maxLines: 4,
	                            wordWrapWidth: 700,
	                            textColor: 0xff101010
	                        }
	                    }
	                }
	            },

	            Button:{
	                y:262, w:w=>w,
	                Background: {
	                    x:w=>w/2, mountX:0.5,
	                },
	                Title: {
	                    y:40,x:w=>w/2, mount:0.5,
	                    text: {
	                        textAlign: 'center',
	                        fontFace: 'OpenSans',
	                        fontSize: 38,
	                        fontStyle: 'bold',
	                        textColor: 0xff00a7e3
	                    }
	                },
	            }
	        }
	    }

	    _init(){
	        this._setState('Inactive');

	        this.tag('Button.Title').on('txLoaded', ()=>{
	            let width = this.tag('Button.Title').finalW+92;
	            this.tag("Button.Background").texture = Lightning.Tools.getRoundRect(width, 68,  35, 4, 0xff1ba7e3, 0, 0);
	        });

	    }

	    set description(value){
	        this.tag('Title.Text').text.text = value;
	        this.tag('Button.Title').text.text = Locale$1.tr.ok;
	    }

	    _handleUp(){

	    }
	    _handleBack(){

	    }
	    _handleDown(){

	    }
	    _handleLeft(){

	    }
	    _handleRight(){

	    }
	    static _states(){
	        return [
	            class Active extends this {
	                _unfocus(){
	                    this._setState('Inactive');
	                }
	                _handleEnter(){
	                    this.signal('OnClick');
	                }
	            },

	            class Inactive extends this {
	                _focus(){
	                    this._setState('Active');
	                }
	            }
	        ]
	    }
	}

	class TopStories extends Lightning.Component {

	    static _template() {
	        return {
	            w: 1920, h: 938, rect: true, color: 0xfff4f8f7,
	            Spinner: {x: 960, y: 470, mount: 0.5, type: SpinningLogo, alpha: 0},
	            Main: {
	                w:w=>w, h:h=>h,
	                PrimaryItem: {
	                    x: 100, y: 43,
	                    type: PrimaryItem,
	                    hidePlayOnFocus: true,
	                    image: Utils.asset('images/live/en.png')
	                },
	                Channel: {
	                    x: 850, y: 50,
	                    text: {
	                        fontFace: 'OpenSans',
	                        fontStyle: '600',
	                        fontSize: 48,
	                        lineHeight: 65
	                    }, color: 0xff101010
	                },
	                ChannelDescription: {
	                    x: 850, y: 109, text: {
	                        fontFace: 'OpenSans', fontSize: 38,
	                        maxLines: 3, wordWrapWidth: 900, lineHeight: 50
	                    }, color: 0xff606060
	                },
	                CurrentProgramTitle: {
	                    x: 850, y: 330, text: {fontFace: 'OpenSans', fontStyle: '600', fontSize: 48}, color: 0xff101010
	                },
	                NextProgram: {
	                    x: 850,
	                    y: 390,
	                    text: {fontFace: 'OpenSans', fontSize: 38},
	                    color: 0xff606060,
	                    alpha: 0,
	                    Title: {
	                        x: 120, text: {fontFace: 'OpenSans', fontSize: 38}, color: 0xff606060
	                    }
	                },
	                LatestBulletins: {
	                    type: List, x: 100, itemSize: 405, storeFocus: true,
	                    y: 470,
	                    // y: 60
	                },
	                LatestStories: {
	                    type: List, x: 100, itemSize: 405, storeFocus: true,
	                    y: 800,
	                    // y: 421
	                },
	            },

	            Error:{
	                alpha:0,y:429,type:Alert, signals:{OnClick:true}
	            }
	        };
	    }

	    _setup() {
	        this._views = {
	            PrimaryItem: this.tag("PrimaryItem"),
	            LatestBulletins: this.tag("LatestBulletins"),
	            LatestStories: this.tag("LatestStories")
	        };
	    }

	    _init() {
	        this._api = this.fireAncestors('$getApi');
	        this.tag('NextProgram').on('txLoaded', ()=>{
	            let width = this.tag('NextProgram').finalW + 20;
	            this.tag('NextProgram.Title').setSmooth('x', width, {duration:0.3});
	        });
	        this._setState("Loading");
	    }

	    _active() {
	        this._setTimeout(0);
	    }

	    _inactive() {
	        this._clearTimeout();

	        this.exit();

	        //reset the live program information
	        this._setLiveProgramInfo();
	    }

	    enter() {
	        this._views.PrimaryItem.transparent  = true;
	        this.fireAncestors('$playLive', "", true);
	    }

	    exit() {
	        this._views.PrimaryItem.transparent  = false;
	        this.fireAncestors("$stopMinimizedPlayback");
	    }

	    _setTimeout(duration) {
	        this._timerId = setTimeout(() => {
	            this._api.getLivePrograms().then( (programs) => {
	                //set the live program information
	                this._setLiveProgramInfo(programs);

	                let nextPingAt = Config.LIVE_PROGRAMS_POLL_FREQUENCY;
	                if(programs) {
	                    var serverTime = new Date().toLocaleString("en-US", {timeZone: "Europe/Paris"});
	                    let currentTime = new Date(serverTime).getTime();
	                    let nextProgramIn = programs.current.end_time - currentTime;
	                    //If the next program starts in less than the poll frequency then adjust the next ping
	                    if(nextProgramIn < Config.LIVE_PROGRAMS_POLL_FREQUENCY) {
	                        nextPingAt = nextProgramIn;
	                    }
	                }
	                this._setTimeout(nextPingAt);
	            }).catch(err => {
	                console.error(err);
	                this._clearTimeout();
	            });
	        }, duration);
	    }

	    _setLiveProgramInfo(programs) {
	        //check for the program info
	        if(programs) {
	            //show the current program title
	            this.tag("CurrentProgramTitle").setSmooth('alpha', 1);
	            this.tag("CurrentProgramTitle").text.text = programs.current.title.toUpperCase();
	            if(programs.next) {
	                //show the next program title
	                this.tag("NextProgram").setSmooth('alpha', 1);
	                this.tag("Title").text.text = programs.next.title.toUpperCase();
	            } else {
	                //Hide the next program info
	                this.tag("NextProgram").setSmooth('alpha', 0);
	            }
	        } else { //Hide the program details
	            this.tag("CurrentProgramTitle").setSmooth('alpha', 0);
	            this.tag("NextProgram").setSmooth('alpha', 0);
	        }
	    }

	    _clearTimeout() {
	        clearTimeout(this._timerId);
	    }

	    _play(item, items) {
	        this.fireAncestors('$play', {
	            items: items,
	            item: item
	        }, true);
	    }

	    reloadPage(){
	        this._setState('Inactive');
	    }

	    static _states() {
	        return [
	            class Loading extends this {
	                $enter() {
	                    this.tag('Spinner').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Main').setSmooth('alpha', 0, {duration:0.3});
	                    this._load();
	                }
	                $exit() {
	                    this.tag('Spinner').setSmooth('alpha', 0, {duration:0.3});
	                    this.tag('Main').setSmooth('alpha', 1, {duration:0.3});

	                    this.fireAncestors('$hideSplash');
	                }
	                _load() {
	                    this.tag('LatestBulletins').title = Locale$1.tr.latestBulletins;
	                    this.tag('LatestStories').title = Locale$1.tr.latestStories;
	                    this.tag('Channel').text.text = Locale$1.tr.live;
	                    this.tag('ChannelDescription').text.text = Locale$1.tr.newsInfo;
	                    this.tag('NextProgram').text.text = Locale$1.tr.next;

	                    let countryCode = Storage.get(Config.SELECTED_LANGUAGE) ? Storage.get(Config.SELECTED_LANGUAGE).countryCode : 'en';
	                    this.tag('PrimaryItem').image = Utils.asset(`images/live/${countryCode?countryCode:'en'}.png`);

	                    const api = this.fireAncestors('$getApi');
	                    api.getLatestBulletins().then((data) => {
	                        this._loadBulletins(data);
	                    }).then(() => api.getLatestStories()).then((stories) => {
	                        this._loadStories(stories);
	                    }).catch(err => {
	                        this._setState('Error');
	                    });
	                }

	                _loadBulletins(data) {
	                    this._views.LatestBulletins.items = data.map((el) => {
	                        return {type: ImageItem, item: el};
	                    });
	                }

	                _loadStories(data) {
	                    this._views.LatestStories.items = data.map((el) => {
	                        return {type: VideoItem, item: el};
	                    });
	                    this._setState("PrimaryItem");
	                    // this._setState("LatestBulletins");
	                }
	                _handleUp(){
	                    //Block the keys till the loading completes
	                }
	                _hanldeDown(){
	                    //Block the keys till the loading completes
	                }
	            },
	            class Idle extends this {
	                _focus() {
	                    this._setState("PrimaryItem");
	                }
	            },
	            class PrimaryItem extends this {
	                $enter() {
	                    this._views.PrimaryItem.transparent  = true;
	                    this.fireAncestors('$playLive', "", true);
	                }

	                $exit() {
	                    this._views.PrimaryItem.transparent  = false;
	                    this.fireAncestors("$stopMinimizedPlayback");
	                }

	                _unfocus() {
	                    this._setState("Idle");
	                }

	                _getFocused() {
	                    return this._views.PrimaryItem;
	                }

	                _handleUp() {
	                    this._setState("Idle");
	                    return false;
	                }

	                _handleDown() {
	                    this._setState("LatestBulletins");
	                }

	                _handleEnter() {
	                    this.fireAncestors('$playLive', this.tag("CurrentProgramTitle").text.text);
	                    this._setState("Idle");
	                }
	            },
	            class LatestBulletins extends this {

	                $exit() {
	                    this._views.LatestBulletins.resetFocus();
	                }

	                _getFocused() {
	                    return this._views.LatestBulletins;
	                }

	                _handleUp() {
	                    this._setState("PrimaryItem");
	                    // this._setState("Idle");
	                    // return false;
	                }

	                _handleDown() {
	                    this._setState("LatestStories");
	                }

	                _handleEnter() {
	                    if (this.tag('LatestBulletins').active.item) {
	                        this._play(this.tag('LatestBulletins').active.item, this.tag('LatestBulletins').items.map(item => item.item));
	                    }
	                }

	            },
	            class LatestStories extends this {

	                $enter() {
	                    this.patch({
	                        smooth: {h: 938 + 370, y: -370}
	                    });
	                }

	                $exit() {
	                    this.patch({
	                        smooth: {h: 938 , y: 0}
	                    });

	                    this._views.LatestStories.resetFocus();
	                }

	                _getFocused() {
	                    return this._views.LatestStories;
	                }

	                _handleUp() {
	                    this._setState("LatestBulletins");
	                }

	                _handleEnter() {
	                    if (this.tag('LatestStories').active.item) {
	                        this._play(this.tag('LatestStories').active.item, this.tag('LatestStories').items.map(item => item.item));
	                    }
	                }
	            },

	            class Error extends this {
	                $enter(){
	                    this.tag('Error').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Main').setSmooth('alpha', 0, {duration:0.3});
	                    this.tag('Error').description = Locale$1.tr.contentProblem;
	                }

	                $exit(){
	                    this.tag('Main').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Error').setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused(){
	                    return this.tag('Error')
	                }

	                OnClick(){
	                    this.application.closeApp();
	                }
	            },

	            class Inactive extends this {
	                _focus(){
	                    this._setState('Loading');
	                }
	            }
	        ];
	    }
	}

	class Album extends Lightning.Component  {

	    static _template() {
	        return {
	            PrimaryItem: {x: 100, y: 53, type: PrimaryItem},
	            Logo: {x: 850, y: 57, alpha: 0},
	            Flex: {
	                x: 850, y: 55,
	                flex: {
	                    direction: 'column', justifyItems: 'flex-start'
	                },
	                Title: {
	                    flexItem: {}, text: {fontFace: 'OpenSans', fontSize: 44, maxLines: 3, wordWrapWidth: 900, lineHeight: 58}, color: 0xff101010
	                },
	                Date: {
	                    flexItem: {}, text: {fontFace: 'OpenSans', fontSize: 38}, color: 0xff909090
	                }
	            },
	            RemainingItems: {
	                type: List, x: 100, y: 501, itemSize: 405, storeFocus: true
	            },
	        };
	    }

	    set items(items) {
	        this._items = items;
	        this._setPrimaryItem(items[0]);
	        this._setListItems(items.slice(1));
	        this._setState("PrimaryItem");
	    }

	    _setPrimaryItem(item) {
	        this.tag("PrimaryItem").imageUrl = item.getPicture(720);
	        this.tag("Title").text.text = item.description || "";
	        this.tag("Date").text.text = item.createdTime ? formatTime(item.createdTime) : '';
	    }

	    _setListItems(items) {
	        this.tag("RemainingItems").items = items.map( el => {
	            return {type: VideoItem, item: el}
	        });
	    }

	    set logo(image) {
	        if(image) {
	            this.patch({
	                Logo: {
	                    texture: Img(image).contain(300, 100),
	                    alpha: 1
	                },
	                Flex: {
	                    smooth: {y: 180}
	                }
	            });
	        } else {
	            this.patch({
	                Logo: {
	                    alpha: 0
	                },
	                Flex: {
	                    smooth: {y: 40}
	                }
	            });
	        }
	    }

	    _play(item, items) {
	        this.fireAncestors('$play', {
	            items: items,
	            item: item
	        }, true);
	    }

	    static _states() {
	        return [
	            class PrimaryItem extends this {
	                _getFocused() {
	                    return this.tag("PrimaryItem");
	                }

	                _handleDown() {
	                    this._setState("RemainingItems");
	                }

	                _handleEnter() {
	                    this._play(this._items[0], this._items);
	                }
	            },
	            class RemainingItems extends this {

	                $exit() {
	                    this.tag("RemainingItems").resetFocus();
	                }

	                _getFocused() {
	                    return this.tag("RemainingItems");
	                }

	                _handleUp() {
	                    this._setState("PrimaryItem");
	                }

	                _handleEnter() {
	                    if (this.tag('RemainingItems').active.item) {
	                        this._play(this.tag('RemainingItems').active.item, this._items);
	                    }
	                }
	            }
	        ];
	    }
	}

	class MostViewed extends Lightning.Component {

	    static _template() {
	        return {
	            w: 1920, h: 938, rect: true, color: 0xfff4f8f7,
	            Main: {
	                w:w=>w, h:h=>h,
	                Spinner: {x: 960, y: 470, mount: 0.5, type: SpinningLogo, alpha: 0},
	                Album: {type: Album, alpha: 0},
	                EmptyList: {
	                    x: w => w / 2, y: h => h / 2, mount: 0.5, alpha: 0,
	                    text: {fontFace: 'OpenSans', textAlign: 'center', fontSize: 38, lineHeight: 32, fontStyle: 'bold', textColor: 0xff101010 }
	                },
	            },

	            Error:{
	                alpha:0, y:429, type:Alert, signals:{OnClick:true}
	            }
	        };
	    }

	    _init() {
	        this._setState("Inactive");
	    }

	    reloadPage(){
	        this._setState('Inactive');
	    }

	    static _states() {
	        return [
	            class Loading extends this {
	                $enter() {
	                    this.tag("Spinner").setSmooth('alpha', 1);
	                    this._load();
	                }

	                $exit() {
	                    this.tag("Spinner").setSmooth('alpha', 0);
	                }

	                _getFocused() {
	                    return this.tag("Spinner");
	                }

	                _load() {
	                    const api = this.fireAncestors('$getApi');
	                    api.getMostViewed().then((data) => {
	                        this._loaded(data);
	                    }).catch(err => {
	                        this._setState('Error');
	                    });
	                }

	                _loaded(data) {
	                    if(data.length > 0){
	                        this.tag("Album").items = data;
	                        this._setState("Album");
	                    }else{
	                        this._setState("EmptyList");
	                    }
	                }
	            },
	            class Album extends this {
	                $enter() {
	                    this.tag("Album").setSmooth('alpha', 1);
	                }

	                $exit() {
	                    this.tag("Album").setSmooth('alpha', 0);
	                }

	                _getFocused() {
	                    return this.tag("Album");
	                }
	            },

	            class EmptyList extends this {
	                $enter() {
	                    this.tag("EmptyList").setSmooth('alpha', 1);
	                    this.tag('EmptyList').text.text = Locale$1.tr.emptyList;
	                    this.fireAncestors('$resetHeader');
	                }

	                _focus(){
	                    this.fireAncestors('$resetHeader');
	                }

	                $exit() {
	                    this.tag("EmptyList").setSmooth('alpha', 0);
	                }

	                _inactive(){
	                    this._setState('Inactive');
	                }
	            },

	            class Error extends this {
	                $enter(){
	                    this.tag('Error').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Main').setSmooth('alpha', 0, {duration:0.3});

	                    this.tag('Error').description = Locale$1.tr.contentProblem;
	                }

	                $exit(){
	                    this.tag('Main').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Error').setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused(){
	                    return this.tag('Error')
	                }

	                _inactive(){
	                    this._setState('Inactive');
	                }

	                OnClick(){
	                    this.fireAncestors('$onError');
	                }
	            },

	            class Inactive extends this {
	                _focus(){
	                    this._setState('Loading');
	                }
	            }
	        ];
	    }
	}

	MostViewed.EMPTYLIST = 'The list is empty.';

	class Grid extends Lightning.Component {
	    static _template() {
	        return {
	            Items: {}
	        };
	    }

	    _init() {
	        this._index = 0;
	        this._pageRow = 1;
	    }

	    /**
	     * Sets the constructor of the grid item.
	     * @param construct Grid item constructor. Must be a sub-class of GridItem.
	     */
	    set construct(construct) {
	        this._construct = construct;
	    }

	    /**
	     * Sets the number of items that can fit in a row based on the Gird width & item width.
	     * @param itemsPerRow items in a row
	     */
	    set itemsPerRow(itemsPerRow) {
	        this._itemsPerRow = itemsPerRow;
	    }

	    /**
	     * Sets the number of rows that can fit in one page.
	     * @warn This is not the total number of rows.
	     * @param rowsPerPage items in a page
	     */
	    set rowsPerPage(rowsPerPage) {
	        this._rowsPerPage = rowsPerPage;
	    }

	    /**
	     * Resets the focus to the first item in the grid
	     */
	    resetFocus() {
	        this._index = 0;
	        this._pageRow = 1;
	        this.patch({Items: {smooth: {y: 0}}});
	    }

	    set items(items) {
	        this._items = items;
	        this.tag("Items").children = items.map((item, index) => {
	            return {
	                type: this._construct,
	                item: item,
	                x: (index % this._itemsPerRow) * this._construct.width,
	                y: Math.floor(index / this._itemsPerRow) * this._construct.height
	            }
	        });
	    }

	    _getRowId(index) {
	        return Math.floor(index / this._itemsPerRow);
	    }

	    get items() {
	        return this._items;
	    }

	    get active() {
	        return this.tag("Items").children[this._index];
	    }

	    _select(index) {
	        if(index < 0 || index > this._items.length - 1){
	            return;
	        }

	        let prevRow = this._getRowId(this._index);
	        let newRow = this._getRowId(index);
	        this._index = index;

	        if(prevRow === newRow) {
	            return;
	        }

	        let yAxisShift;

	        //handle key down
	        if(prevRow < newRow) {
	            //if the current row is maximum in the page, then shift y
	            if(this._pageRow === this._rowsPerPage) {
	                yAxisShift = (newRow - (this._rowsPerPage - 1)) * this._construct.height * -1;
	            } else {
	                this._pageRow++;
	            }
	        } else { //handle key up
	            if(this._pageRow === 1) { //if the current row is minimum in the page, then shift y
	                yAxisShift = newRow * this._construct.height * -1;
	            } else {
	                this._pageRow--;
	            }
	        }

	        if(yAxisShift != undefined) {
	            this.patch({Items: {smooth: {y: yAxisShift}}});
	        }

	    }

	    _handleLeft() {
	        const index = this._index;
	        //allow the left navigation only if the item is not the first item in the row
	        if (index % this._itemsPerRow !== 0) {
	            this._select(index - 1);
	        } else {
	            return false;
	        }
	    }

	    _handleRight() {
	        const index = this._index;
	        //allow the right navigation only if the item is not the last item in the row
	        if(index % this._itemsPerRow !== (this._itemsPerRow - 1)) {
	            this._select(index + 1);
	        } else {
	            return false;
	        }
	    }

	    _handleUp() {
	        let index = this._index;

	        if (index - this._itemsPerRow >= 0) {
	            index -= this._itemsPerRow;
	        } else {
	            this._index = 0;
	            return false;
	        }
	        this._select(index);
	    }

	    _handleDown() {
	        let index = this._index;
	        if (index + this._itemsPerRow > this._items.length - 1) {
	            index = this._items.length - 1;
	        } else {
	            index += this._itemsPerRow;
	        }
	        this._select(index);
	    }

	    _getFocused() {
	        return this.active;
	    }
	}

	class GridItem extends Lightning.Component{

	    /**
	     * Sub class must return the width of the grid item.
	     * @returns {number} grid item width
	     */
	    static get width() {
	        return 0;
	    }

	    /**
	     * Sub class must return the height of the grid item.
	     * @returns {number} grid item height
	     */
	    static get height() {
	        return 0;
	    }

	}

	class ImageItem$1 extends GridItem {
	    static _template() {
	        return {
	            Wrapper: {
	                w: 382, h: 250,
	                Image: {
	                    Shadow: {
	                        x: -28, y: 174,
	                        color: 0x35000000,
	                        texture: Lightning.Tools.getShadowRect(368, 6, 0, 3, 30),
	                    },
	                    Picture: {w: 370, h: 205},
	                    DefaultImage: {w: 370, h: 205, src: Utils.asset('images/default-image.png')},
	                    Border: {
	                        alpha: 0,
	                        Border: {
	                            type: Lightning.components.BorderComponent,
	                            borderWidth: 6,
	                            colorBorder: 0xff00a7e3,
	                            w: 370,
	                            h: 205
	                        }
	                    },
	                },
	                Title: {
	                    y: 215,
	                    text: {fontSize: 24, fontFace: 'OpenSans', fontStyle: 'bold', maxLines: 1, wordWrapWidth: 300},
	                    color: 0xff101010
	                }
	            }
	        };

	    }

	    _init() {
	        this.tag('Picture').on('txLoaded', () => {
	            this.patch({
	                Wrapper: {
	                    Image: {DefaultImage: {smooth: {alpha: 0}}},
	                }
	            });
	        });
	    }

	    static get width() {
	        return 450;
	    }

	    static get height() {
	        return 285;
	    }

	    set item(v) {
	        this._item = v;
	        this.patch({
	            Wrapper: {
	                Image: {Picture: {texture: Img(v.getPicture(388)).cover(370,205)}},
	                Title: {
	                    text: {text: v.title.toUpperCase()}
	                }
	            }
	        });
	    }

	    get item() {
	        return this._item;
	    }

	    _focus() {
	        this.patch({
	            Wrapper: {
	                Image: {
	                    Shadow: {smooth: {alpha: 0}},
	                    Border: {smooth: {alpha: 1}},
	                    //smooth: {scale: 1.1}
	                },
	                Title: {
	                    color: 0xffff00a7e3
	                },
	                smooth: {scale: 1.1}
	            }
	        });
	    }

	    _unfocus() {
	        this.patch({
	            Wrapper: {
	                Image: {
	                    Border: {smooth: {alpha: 0}},
	                    Shadow: {smooth: {alpha: 1}},
	                    smooth: {scale: 1}
	                },
	                Title: {
	                    color: 0xff101010
	                },
	                smooth: {scale: 1}
	            }
	        });
	    }
	}

	class Shows extends Lightning.Component {

	    static _template() {
	        return {
	            w: 1920, h: 938, rect: true, color: 0xfff4f8f7,
	            Main: {
	                w: w=>w, h:h=>h,
	                Spinner: {x: 960, y: 470, mount: 0.5, type: SpinningLogo},
	                Shows: {x: 100, y: 50, type: Grid, construct: ImageItem$1, itemsPerRow: 4, rowsPerPage: 3, alpha: 0},
	                ShowContent: {type: Album, alpha: 0},
	                EmptyList: {
	                    x: w => w / 2, y: h => h / 2, mount: 0.5, alpha: 0,
	                    text: {fontFace: 'OpenSans', textAlign: 'center', fontSize: 38, lineHeight: 32, fontStyle: 'bold', textColor: 0xff101010}
	                },
	                transitions:{alpha:{duration:0.3}},
	            },
	            Error:{
	                alpha:0, y:429, type:Alert,signals:{OnClick:true}
	            }
	        };
	    }

	    _init() {
	        this._shows = new Map();
	        this._showCategories = new Map();
	        this._setState("Inactive");
	    }

	    reset() {
	        this._reset();
	    }

	    reloadPage(){
	        this._setState('Inactive');
	    }

	    static _states() {
	        return [
	            class Loading extends this {
	                $enter() {
	                    this.tag("Spinner").setSmooth('alpha', 1);
	                    this._load();
	                }

	                $exit() {
	                    this.tag("Spinner").setSmooth('alpha', 0);
	                }

	                _getFocused() {
	                    return this.tag("Spinner");
	                }

	                _load() {
	                    const api = this.fireAncestors('$getApi');
	                    api.getShowCategories().then((data) => {
	                        this._loaded(data);
	                    }).catch(reject=>{this._setState('Error');});
	                }

	                _loaded(data) {
	                    if(data.length > 0) {
	                        data.forEach(item => {
	                            this._showCategories.set(item.nid, item);
	                        });
	                        this.tag("Shows").items = data;
	                        this._setState("Shows");
	                    }else {
	                        this._setState("EmptyList");
	                    }
	                }
	            },

	            class EmptyList extends this {
	                $enter() {
	                    this.tag("EmptyList").setSmooth('alpha', 1);
	                    this.tag('EmptyList').text.text = Locale$1.tr.emptyList;
	                    this.fireAncestors('$resetHeader');
	                }

	                $exit() {
	                    this.tag("EmptyList").setSmooth('alpha', 0);
	                }

	                _focus(){
	                    this.fireAncestors('$resetHeader');
	                }

	                _inactive(){
	                    this._setState('Inactive');
	                }
	            },

	            class Shows extends this {
	                $enter() {
	                    this.tag("Shows").setSmooth('alpha', 1);
	                }

	                $exit() {
	                    this.tag("Shows").setSmooth('alpha', 0);
	                }

	                _changedState({newState, prevState}, args) {
	                    if(newState === "Shows" && prevState === "Shows.ShowContent" &&
	                        args && args.reset) {
	                        //reset the shows grid focus to first item
	                        this.tag("Shows").resetFocus();
	                    }
	                }

	                _handleEnter() {
	                    this._selectedNid = this.tag("Shows").active.item.nid;
	                    if(this._shows.get(this._selectedNid)) {
	                        this._setState("Shows.ShowContent");
	                    } else {
	                        this._setState("Shows.Loading");
	                    }
	                }

	                _getFocused() {
	                    return this.tag("Shows");
	                }

	                static _states() {
	                    return [
	                        class Loading extends this {
	                            $enter() {
	                                this.tag("Shows").setSmooth('alpha', 0);
	                                this.tag("Spinner").setSmooth('alpha', 1);
	                                this._load();
	                            }

	                            $exit() {
	                                this.tag("Spinner").setSmooth('alpha', 0);
	                                this.tag("ShowContent").setSmooth('alpha', 0);

	                            }

	                            _getFocused() {
	                                return this.tag("Spinner");
	                            }

	                            _load() {
	                                const api = this.fireAncestors('$getApi');
	                                api.getShows(this._selectedNid).then((data) => {
	                                    this._loaded(data);
	                                }).catch(err => {
	                                    if(this._getState() === 'Shows.Loading' || this._getState() === 'Shows.ShowContent')
	                                        this._setState('Shows.Error');
	                                });
	                            }

	                            _loaded(data) {
	                                if(data && data.length >0) {
	                                    this._shows.set(this._selectedNid, data);
	                                    this._setState("Shows.ShowContent");
	                                }else {
	                                    this._setState("Shows.EmptyList");
	                                }
	                            }

	                            _handleBack() {
	                                this.tag("Shows").setSmooth('alpha', 1);
	                                this._setState("Shows");
	                            }
	                        },
	                        class ShowContent extends this {
	                            $enter() {
	                                this.tag("Shows").setSmooth('alpha', 0);
	                                this.tag("ShowContent").items = this._shows.get(this._selectedNid);
	                                this.tag("ShowContent").logo = this._showCategories.get(this._selectedNid).headerImage;
	                                this.tag("ShowContent").setSmooth('alpha', 1);
	                            }

	                            $exit() {
	                                this.tag("ShowContent").setSmooth('alpha', 0);
	                                this.tag("Shows").setSmooth('alpha', 1);
	                            }

	                            _reset() {
	                                this._setState('Shows', [{reset: true}]);
	                            }

	                            _handleBack() {
	                                this._setState("Shows");
	                            }

	                            _getFocused() {
	                                return this.tag("ShowContent");
	                            }
	                        },

	                        class EmptyList extends this {
	                            $enter() {
	                                this._shows.delete(this._selectedNid);
	                                this.tag("EmptyList").setSmooth('alpha', 1);
	                                this.tag('EmptyList').text.text = Locale$1.tr.emptyList;
	                            }

	                            $exit() {
	                                this.tag("ShowContent").setSmooth('alpha', 0);
	                                this.tag("Shows").setSmooth('alpha', 1);
	                                this.tag("EmptyList").setSmooth('alpha', 0);
	                            }

	                            _handleBack() {
	                                this._setState("Shows");
	                            }

	                            _inactive(){
	                                this._setState("Shows");
	                            }
	                        },

	                        class Error extends this {
	                            $enter(){
	                                this._shows.delete(this._selectedNid);
	                                this.tag('Error').setSmooth('alpha', 1, {duration:0.3});
	                                this.tag('Main').setSmooth('alpha', 0, {duration:0.3});

	                                this.tag('Error').description = Locale$1.tr.contentProblem;
	                            }

	                            $exit(){
	                                this.tag("Spinner").setSmooth('alpha', 0);
	                                this.tag("ShowContent").setSmooth('alpha', 0);
	                                this.tag("Shows").setSmooth('alpha', 1);
	                                this.tag('Main').setSmooth('alpha', 1, {duration:0.3});
	                                this.tag('Error').setSmooth('alpha', 0, {duration:0.3});
	                            }

	                            _getFocused(){
	                                return this.tag('Error')
	                            }

	                            OnClick(){
	                                this._setState("Shows");
	                            }
	                        },
	                    ];
	                }
	            },

	            class Error extends this {
	                $enter(){
	                    this.tag('Error').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Main').setSmooth('alpha', 0, {duration:0.3});

	                    this.tag('Error').description = Locale$1.tr.contentProblem;
	                }

	                $exit(){
	                    this.tag('Main').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Error').setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused(){
	                    return this.tag('Error')
	                }

	                _inactive(){
	                    this._setState('Inactive');
	                }

	                OnClick(){
	                    this.fireAncestors('$onError');
	                }
	            },

	            class Inactive extends this {
	                _focus(){
	                    this._setState('Loading');
	                }
	            }
	        ];
	    }
	}

	class Scrollbar extends Lightning.Component {

	    constructor(stage) {
	        super(stage);
	        this._direction = 'vertical';
	        this._colors = {
	            background : 0xffd8d8d8,
	            scrollerFocused : 0xff00a7e3,
	            scrollerUnfocused : 0xff606060
	        };
	    }

	    static _template() {
	        return {
	            Background: {
	                h: h=>h, w: w=>w, rect: true, color: 0xffd8d8d8
	            },
	            Scroller: {
	                h: h=>h, w: w=>w, rect: true,
	            }
	        };
	    }

	    _init() {
	        this._setState("Inactive");
	    }

	    /**
	     * Set the scroll bar direction.
	     * @param v vertical or horizontal
	     */
	    set direction(v) {
	        this._direction = v;
	    }

	    /**
	     * Set the required colors for the scroll bar.
	     * @param background scroll bar color
	     * @param scrollerFocused scroller color when focused
	     * @param scrollerUnfocused scroller color when it is out of focus
	     */
	    set colors({background = null, scrollerFocused = null, scrollerUnfocused = null}) {
	        this._colors.background = background || this._colors.background;
	        this._colors.scrollerFocused = scrollerFocused || this._colors.scrollerFocused;
	        this._colors.scrollerUnfocused = scrollerUnfocused || this._colors.scrollerUnfocused;

	        this.tag("Background").patch({color: this._colors.background});
	    }

	    /**
	     * Set the view sizes to which you want to apply scrolling.
	     * @param visible view visible size
	     * @param total view total size
	     */
	    set sizes({visible , total}) {
	        this._visibleSize = visible;
	        this._totalScrollSize = total;

	        //current scroller position
	        this._scrollerPosition = 0;

	        //number of scrolls possible with scrolling only 80% of the view every time
	        this._totoalScrolls = this._totalScrollSize / (0.8 * this._visibleSize);

	        //scroller height
	        this._scrollerSize = this._scrollbarTotalSize / this._totoalScrolls;

	        let scrollerSizeProperty = this._direction === 'vertical' ? 'h' : 'w';
	        this.tag("Scroller").patch({smooth: {[scrollerSizeProperty]: this._scrollerSize}});

	        this._apply();
	    }

	    get _scrollbarTotalSize() {
	        return this.tag("Background")[this._direction === 'vertical' ? 'finalH' : 'finalW'];
	    }

	    get _scrollbarStart() {
	        return this.tag("Background")[this._direction === 'vertical' ? 'finalY' : 'finalX'];
	    }

	    get _scrollerStartPosition() {
	        return this._scrollbarStart + (this._scrollerPosition * this._scrollerSize);
	    }

	    get _viewStartPosition() {
	        return ((Math.floor(this._totalScrollSize / this._totoalScrolls)) * this._scrollerPosition) * -1;
	    }

	    get _axis() {
	        return this._direction === 'vertical' ? 'y' : 'x';
	    }

	    _handleNext() {
	        let scrollerPosition = this._scrollerPosition;
	        if(this._scrollerPosition < this._totoalScrolls - 1) {
	            scrollerPosition++;
	            if(scrollerPosition > this._totoalScrolls - 1) {
	                scrollerPosition = scrollerPosition - (scrollerPosition - (this._totoalScrolls -1));
	            }
	            this._scrollerPosition = scrollerPosition;
	            this._apply();
	        } else {
	            return false;
	        }

	    }

	    _handlePrevious() {
	        if(this._scrollerPosition > 0) {
	            this._scrollerPosition--;
	            this._scrollerPosition = this._scrollerPosition < 0 ? 0 : this._scrollerPosition;
	            this._apply();
	        } else {
	            return false;
	        }
	    }

	    _apply() {
	        let scrollerStart = this._scrollerStartPosition;
	        let viewStart = this._viewStartPosition;
	        let axis  = this._axis;

	        this.tag("Scroller").patch({smooth: {[axis]: scrollerStart}});
	        this.signal('scrollTo', viewStart );
	    }

	    static _states() {
	        return [
	            class Inactive extends this {
	                $enter() {
	                    this.patch({
	                        Scroller: {color: this._colors.scrollerUnfocused}
	                    });
	                }

	                _focus() {
	                    this._setState("Active");
	                }

	            },
	            class Active extends this {
	                $enter() {
	                    this.patch({
	                        Scroller: {color: this._colors.scrollerFocused}
	                    });
	                }

	                _unfocus() {
	                    this._setState("Inactive");
	                }

	                _handleDown() {
	                    if(this._direction === 'vertical') {
	                        return this._handleNext();
	                    } else {
	                        return false;
	                    }
	                }

	                _handleUp() {
	                    if(this._direction === 'vertical') {
	                        return this._handlePrevious();
	                    } else {
	                        return false;
	                    }
	                }

	                _handleRight() {
	                    if(this._direction !== 'vertical') {
	                        return this._handleNext();
	                    } else {
	                        return false;
	                    }
	                }

	                _handleLeft() {
	                    if(this._direction !== 'vertical') {
	                        return this._handlePrevious();
	                    } else {
	                        return false;
	                    }
	                }

	            }
	        ];
	    }

	}

	class About extends Lightning.Component {

	    static _template() {
	        return {
	            w: 1920, h: 938, rect: true, color: 0xfff4f8f7,
	            Publisher: {
	                x: 150, y: 100,
	                Title: {
	                    text: {fontFace: 'OpenSans', fontStyle: '600', fontSize: 32, lineHeight: 40},
	                    color: 0xff101010
	                },
	                Description: {
	                    y: 70, w: 600,
	                    Text: {
	                        text: {
	                            fontFace: 'OpenSans',
	                            fontSize: 22,
	                            lineHeight: 40,
	                            wordWrapWidth: 550,
	                            textColor: 0xff606060
	                        }
	                    }
	                }
	            },

	            Terms: {
	                x: 850, y: 100, w: 800,
	                Title: {
	                    text: {fontFace: 'OpenSans', fontStyle: '600', fontSize: 32, lineHeight: 43},
	                    color: 0xff101010
	                },
	                Clipper: {
	                    y: 70, w: 800, h: 780, clipping: true,
	                    Description: {
	                        w: 800,
	                        flex: {direction: "column", justifyItems: "flex-start"}
	                    }
	                }
	            },
	            Scrollbar: {
	                x: 1760, y: 100, w: 10, h: 720, type: Scrollbar, signals : {'scrollTo': true}
	            }
	        }
	    }

	    _init() {
	        this._frameListener = () => {
	            let newHeight = this.tag("Terms.Clipper.Description").finalH;
	            if(newHeight !== this._termsTotalHeight) {
	                this._termsTotalHeight = newHeight;
	                this._termsVisibleHeight = this.tag("Terms.Clipper").finalH;
	                this._setScrollerSizes();
	            }
	        };

	        this._setState("");
	    }

	    _updateUI(){
	        let publisher = Locale$1.tr.publisher.toString();
	        let termsAndConditionsTitle = Locale$1.tr.termsAndConditions.toString();

	        if(publisher === this.tag('Publisher.Title').text.text &&
	            this.tag('Terms.Title').text.text === termsAndConditionsTitle){
	            return;
	        }

	        this.tag('Publisher.Title').text.text = publisher;
	        let publisherDescription = Locale$1.tr.publisherDescription;
	        this.tag('Terms.Title').text.text = termsAndConditionsTitle;
	        let termsAndConditions = Locale$1.tr.termsAndConditionsDescription.toString().split('\n');

	        //add publisher description
	        this.tag("Publisher.Description.Text").text.text = publisherDescription;


	        //add terms & conditions description
	        let termsChildren = [];
	        termsAndConditions.forEach((item, blockIndex) => {
	            let marginTop = (blockIndex === 0) ? 0 : 30;
	            termsChildren.push({
	                flexItem: {marginTop: marginTop},
	                text: {text: item, fontFace: 'OpenSans', fontSize: 22, lineHeight: 40, wordWrapWidth: 750}, color: 0xff606060});
	        });
	        this.tag("Terms.Clipper.Description").children = termsChildren;

	    }

	    _setScrollerSizes() {
	        this.tag("Scrollbar").sizes = {visible: this._termsVisibleHeight, total: this._termsTotalHeight};
	    }

	    _active() {
	        this._updateUI();
	        this.stage.on('frameStart', this._frameListener);
	        this._setState("Active");
	    }

	    _inactive() {
	        this.stage.removeListener('frameStart', this._frameListener);
	        //this._setState(""); //Todo: Does not work if app is closed on this state. app will break
	    }

	    static _states() {
	        return [
	            class Active extends this {
	                _getFocused() {
	                    return this.tag("Scrollbar");
	                }

	                scrollTo(position) {
	                    this.tag("Terms.Clipper.Description").patch({smooth : {y: position}});
	                }

	            }
	        ];
	    }
	}

	class LanguageItem extends Lightning.Component {

	    static _template() {
	        return {
	            Image:{
	                Unfocus:{src:Utils.asset('images/france-24-unfocus-logo.png')},
	                Focus:{alpha:0,src:Utils.asset('images/france-24-logo.png')},
	                transitions:{alpha:{duration:0.3}}
	            },
	            Title: {y:275, w:270, text: {fontFace: 'Amri', textAlign:'center', fontSize: 60, lineHeight:32, fontStyle:'bold', textColor: 0xff909090}},
	        }
	    }

	    set item(item){
	        this.tag('Title').text.text = item.title;
	    }

	    set selection(t){
	        this._selected = t;
	        this.tag('Title').setSmooth('text.textColor', this._selected?0xff00a7e3:0xff909090, {duration:0.3});
	        this._updateImageView(this._selected);
	    }

	    _focus() {
	        this._updateImageView(true);
	        this.tag('Title').setSmooth('text.textColor', 0xff00a7e3, {duration:0.3});
	    }

	    _unfocus() {
	        this._updateImageView(false);
	        this.tag('Title').setSmooth('text.textColor', this._selected?0xff101010:0xff909090, {duration:0.3});
	    }

	    _updateImageView(value){
	        this.tag('Focus').setSmooth('alpha', value?1:0, {duration:0.3});
	    }
	}

	class Languages extends Lightning.Component {
	    static _template(){
	        return {
	            w: 1920, h: 938, rect: true, color: 0xfff4f8f7,
	            Layout:{
	                w:1920, y:238,
	                Items:{
	                    w:w=>w,
	                    flex:{direction: 'row', wrap:false, justifyContent:'center', alignItems:'center'}
	                }
	            },

	            Alert:{
	                alpha:0, y:429, type:Alert,signals:{OnClick:true}
	            }
	        }
	    }


	    _init() {
	        this._focusIdx = 0;
	        this._selectedIdx = 0;

	        this.tag("Items").children = this._getItems().map((item, index) => {
	            if(item.selection){
	                this._focusIdx = index;
	                this._selectedIdx = index;
	            }

	            return {
	                w:270,
	                flexItem:{marginLeft:40, marginRight:40},
	                type: LanguageItem,
	                item : item,
	                selection:item.selection,
	            };
	        });


	        this._setState('List');
	    }

	    _getItems(){
	        let value = Storage.get(Config.SELECTED_LANGUAGE);
	        value = value ? value : Languages.SUPPORTED_LANGUAGES[1];

	        return Languages.SUPPORTED_LANGUAGES.map((item, index)=> {
	            item.selection = item.countryCode === value.countryCode;
	            return item;
	        });
	    }

	    static _states(){
	        return [
	            class List extends this {

	                _getFocused() {
	                    return this.tag("Items").children[this._focusIdx];
	                }

	                _handleRight() {
	                    if(this._focusIdx < Languages.SUPPORTED_LANGUAGES.length - 1) {
	                        this._focusIdx++;
	                    }
	                }

	                _handleLeft() {
	                    if(this._focusIdx > 0) {
	                        this._focusIdx--;
	                    }
	                }

	                _handleDown() {
	                    /*Reset the focused to selected. So that the next time when the focus is gained by Menu,
	                    then the selected will be highlighted.
	                     */
	                    this._focusIdx = this._selectedIdx;
	                    return false;
	                }

	                _handleEnter(){
	                    let value = Storage.get(Config.SELECTED_LANGUAGE);
	                    if(!value || Languages.SUPPORTED_LANGUAGES[this._focusIdx].countryCode !== value.countryCode){
	                        this._setState('Alert');
	                    }
	                }
	            },

	            class Alert extends this {
	                $enter(){
	                    this.tag('Alert').setSmooth('alpha', 1, {duration:0.3});
	                    this.tag('Layout').setSmooth('alpha', 0.1, {duration:0.3});
	                    this.tag('Alert').description = Locale$1.tr.languageChanged;
	                }

	                $exit(){
	                    this.tag('Alert').setSmooth('alpha', 0, {duration:0.3});
	                    this.tag('Layout').setSmooth('alpha', 1, {duration:0.3});
	                }

	                _getFocused() {
	                    return this.tag("Alert")
	                }

	                OnClick(){

	                    this._selectedIdx = this._focusIdx;
	                    this.tag("Items").children.forEach((item, index) => {
	                        item.selection = (this._selectedIdx === index);
	                    });

	                    let selectedLanuage = Languages.SUPPORTED_LANGUAGES[this._focusIdx];
	                    Storage.set(Config.SELECTED_LANGUAGE, selectedLanuage);
	                    Locale$1.setLanguage(selectedLanuage.countryCode);

	                    this.fireAncestors("$onLanguageChange");
	                    this.signal('languageChanged');

	                    this._setState('List');
	                }
	            }
	        ]
	    }

	}

	Languages.SUPPORTED_LANGUAGES = [
	    {title: "français", countryCode:'fr'},
	    {title: "english", countryCode:'en'},
	    {title: "español", countryCode:'es'},

	    //As the Arabic language rendering(rtl) doesn't spport in the lightning.
	    //Enabled once we get the support
	    //While enabling update the live message in the home screen to below
	    //International News 24/7 in French, English, Arabic and Spanish
	    // {title: "عربى", countryCode:'ar'},
	];

	class Content extends Lightning.Component {

	    static _template() {
	        return {
	            TopStories: {type: TopStories, alpha: 0},
	            MostViewed: {type: MostViewed, alpha: 0},
	            Shows: {type: Shows, alpha: 0},
	            Languages:{type: Languages, alpha: 0, signals:{languageChanged:true}},
	            About: {type: About, alpha: 0},

	            transitions:{alpha:{duration:0.3}}
	        };
	    }

	    _init() {
	        this._mapping = {
	            "topStories": 'TopStories',
	            "mostViewed": 'MostViewed',
	            "shows": 'Shows',
	            "languages":'Languages',
	            "about": 'About'
	        };

	        this._setState("TopStories");
	    }

	    select(view) {
	        if(this._mapping.hasOwnProperty(view)) {
	            this._setState(this._mapping[view]);
	        }
	    }

	    reloadPage() {
	        this.tag('TopStories').reloadPage();
	        this.tag('MostViewed').reloadPage();
	        this.tag('Shows').reloadPage();
	    }


	    static _states() {
	        return [
	            class TopStories extends this {
	                $enter() {
	                    this.tag("TopStories").setSmooth('alpha', 1, {duration:0.3});
	                }

	                $exit() {
	                    this.tag("TopStories").setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused() {
	                    return this.tag("TopStories");
	                }
	            },
	            class MostViewed extends this {

	                $enter() {
	                    this.tag("MostViewed").setSmooth('alpha', 1, {duration:0.3});
	                }

	                $exit() {
	                    this.tag("MostViewed").setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused() {
	                    return this.tag("MostViewed");
	                }

	                _handleBack(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }

	                $onError(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }
	            },

	            class Shows extends this {
	                $enter() {
	                    this.tag("Shows").setSmooth('alpha', 1, {duration:0.3});
	                }

	                $exit() {
	                    this.tag("Shows").setSmooth('alpha', 0, {duration:0.3});
	                    this.tag("Shows").reset();
	                }

	                _getFocused() {
	                    return this.tag("Shows");
	                }

	                _handleBack(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }

	                $onError(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }
	            },

	            class Languages extends this {
	                $enter() {
	                    this.tag("Languages").setSmooth('alpha', 1, {duration:0.3});
	                }

	                $exit() {
	                    this.tag("Languages").setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused() {
	                    return this.tag("Languages");
	                }

	                _handleBack(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }

	                languageChanged(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }

	                $onLanguageChange(){
	                    this.reloadPage();
	                }
	            },

	            class About extends this {

	                $enter() {
	                    this.tag("About").setSmooth('alpha', 1, {duration:0.3});
	                }

	                $exit() {
	                    this.tag("About").setSmooth('alpha', 0, {duration:0.3});
	                }

	                _getFocused() {
	                    return this.tag("About");
	                }

	                _handleBack(){
	                    this.signal('resetHeader');
	                    this._setState('TopStories');
	                }
	            }
	        ];
	    }
	}

	class Main extends Lightning.Component {

	    static _template() {
	        return {
	            Content: {x:0, y:142, w: w=>w, type: Content, signals:{resetHeader:true, languageChange:true}},
	            Header: {type: Header, zIndex:2}
	        };
	    }

	    _init() {
	        this._setState("Content");
	    }

	    static _states() {
	        return [
	            class Header extends this {
	                _getFocused() {
	                    return this.tag("Header");
	                }

	                $select(item) {
	                    this.tag("Content").select(item);
	                    this._setState("Content");
	                }

	                _handleDown() {
	                    this._setState("Content");
	                }
	            },
	            class Content extends this {
	                _getFocused() {
	                    return this.tag("Content");
	                }

	                _handleUp() {
	                    this._setState("Header");
	                }

	                resetHeader(){
	                    this.tag("Header").reset();
	                }

	                languageChange(){
	                    this.tag("Header").reset();
	                }

	                $resetHeader(){
	                    this._setState("Header");
	                }
	            }
	        ];
	    }
	}

	class MediaItem {
	    constructor() {
	    }

	    set title(title) {
	        this._title = title;
	    }

	    get title() {
	        return this._title;
	    }

	    set description(description) {
	        this._description = description;
	    }

	    get description() {
	        return this._description;
	    }

	    set pictures(pictures) {
	        this._pictures = pictures;
	    }

	    get pictures() {
	        return this._pictures;
	    }

	    set headerImage(url) {
	        this._header_image = url;
	    }

	    get headerImage() {
	        return this._header_image;
	    }

	    set videos(videos) {
	        this._videos = videos;
	    }

	    get videos() {
	        return this._videos;
	    }

	    set createdTime(createdTime) {
	        this._createdTime = createdTime ? parseInt(createdTime) || 0 : 0;
	    }

	    get createdTime() {
	        return this._createdTime;
	    }

	    set nid(nid) {
	        this._nid = nid;
	    }

	    get nid() {
	        return this._nid;
	    }

	    /**
	     * Get a picture that matches a certain width or height
	     * @param w
	     * @param h
	     * @returns {*}
	     */
	    getPicture(w = null, h = null) {
	        let pictures = this._pictures;

	        if (!pictures.length) {
	            return false;
	        }
	        if (!w && !h) {
	            return pictures[0];
	        } else {
	            const val = w ? w : h;
	            const match = pictures.filter(p => p[w ? 'width' : 'height'] === val);

	            if (match.length) {
	                return match[0].url;
	            } else {
	                return pictures[0].url;
	            }
	        }
	    }

	    get video() {
	        return this.videos[0].url;
	    }

	    getMediaplayerItem() {
	        return this._mediaplayerItem ? this._mediaplayerItem :
	            this._mediaplayerItem = {title: this.title, description: this.description, stream: {link: this.video}}
	    }

	    /**
	     * Constructs a Media item object from the given news bulletin item.
	     *
	     * @param channel {object} bulletin item.
	     * @returns {MediaItem} Media item
	     */
	    static fromBulletinItem(channel) {
	        let mediaItem = new MediaItem();
	        mediaItem.title = channel.title ? channel.title.trim() : channel.title ;
	        mediaItem.pictures = MediaItem._getPictures(channel.images.formats);
	        mediaItem.videos = channel.videos[0].formats;
	        return mediaItem;
	    }

	    /**
	     * Constructs a Media item object from the given show category.
	     *
	     * @param show {object} show category item
	     * @returns {MediaItem} Media item
	     */
	    static fromShowCategoryItem(show) {
	        let mediaItem = new MediaItem();
	        mediaItem.title = show.title ? show.title.trim() : show.title;
	        mediaItem.nid = show.nid;
	        mediaItem.pictures = MediaItem._getPictures(show.images.formats);
	        mediaItem.headerImage = MediaItem._getHeaderImage(show);
	        return mediaItem;
	    }

	    /**
	     * Constructs a Media item object from the given show content item.
	     *
	     * @param show {object} show content item
	     * @returns {MediaItem} Media item
	     */
	    static fromShowContentItem(show) {
	        let mediaItem = new MediaItem();
	        mediaItem.title = show.surtitle ? show.surtitle.trim() : show.surtitle;
	        mediaItem.description = show.title ? show.title.trim() : show.title;
	        mediaItem.createdTime = show.created;
	        mediaItem.pictures = MediaItem._getPictures(show.images.formats);
	        mediaItem.videos = Api.filterMP4(show.videos[0].formats);
	        return mediaItem;
	    }

	    /**
	     * Constructs a Media item object from the given most viewed item.
	     *
	     * @param mostViewed {object} most viewed item
	     * @returns {MediaItem} Media item
	     */
	    static fromMostViewedItem(mostViewed) {
	        let mediaItem = new MediaItem();
	        mediaItem.title = mostViewed.title ? mostViewed.title.trim() : mostViewed.title;
	        mediaItem.description = mostViewed.title ? mostViewed.title.trim() : mostViewed.title;
	        mediaItem.createdTime = mostViewed.created;
	        mediaItem.pictures = MediaItem._getPictures(mostViewed.images.formats);
	        mediaItem.videos = Api.filterMP4(mostViewed.formats);
	        return mediaItem;
	    }

	    static _getPictures(formats) {
	        return formats.map(format => {
	            let dimensions = format.code.split('x');
	            let w = 0, h = 0;
	            if(dimensions.length == 2) {
	                w = parseInt(dimensions[0]);
	                h = parseInt(dimensions[1]);
	            }
	            return {width: w, height: h, url: format.url}
	        }).sort((a,b) => b.width - a.width);
	    }

	    static _getHeaderImage(show) {
	        if(show.header_images && show.header_images.formats &&
	            show.header_images.formats.length) {
	            return show.header_images.formats.filter(format => format.code === 'original')[0].url;
	        }
	        return "";
	    }

	}

	class Api {

	    constructor() {
	        this._keys = {
	            token: 'c7d743bf-dff4-4884-8038-18056c1f7380'
	        };

	        this._endpoints = {
	            fr: {
	                latestBulletins: `http://apis.france24.com/products/get_product/ott_last_news_fr?token_application=${this._keys.token}`,
	                showCategories: `http://apis.france24.com/products/get_product/51d69932-e4de-11e7-a95c-005056a90194?token_application=${this._keys.token}`,
	                showContent: `http://apis.france24.com/products/get_product/5f211206-752b-11e8-92b1-005056a945c4?token_application=${this._keys.token}`,
	                mostViewed: `http://apis.france24.com/products/get_product/787aaa5e-b695-11e8-adf8-005056a945c4?token_application=${this._keys.token}`,

	                latestStories: `http://apis.france24.com/products/get_product/51782356-73d2-11e8-b324-005056a945c4?token_application=${this._keys.token}`,
	                liveTvPrograms: `http://apis.france24.com/products/get_product/7a45c188-752d-11e8-8f5e-005056bf3e3d?token_application=${this._keys.token}`,
	            },

	            en:{
	                latestBulletins: `http://apis.france24.com/products/get_product/ott_last_news_en?token_application=${this._keys.token}`,
	                showCategories: `http://apis.france24.com/products/get_product/a2332ae0-88d3-43df-9c48-c9d9e1f6e5ef?token_application=${this._keys.token}`,
	                showContent: `http://apis.france24.com/products/get_product/d3edba4a-793f-11e8-8bf1-005056a945c4?token_application=${this._keys.token}`,
	                mostViewed: `http://apis.france24.com/products/get_product/7750926a-b695-11e8-9e60-005056bf3e3d?token_application=${this._keys.token}`,

	                latestStories: `http://apis.france24.com/products/get_product/b5f278ea-73d2-11e8-9e7f-005056a945c4?token_application=${this._keys.token}`,
	                liveTvPrograms: `http://apis.france24.com/products/get_product/fe445440-7537-11e8-87cc-005056a945c4?token_application=${this._keys.token}`
	            },

	            ar:{
	                latestBulletins: `http://apis.france24.com/products/get_product/ott_last_news_ar?token_application=${this._keys.token}`,
	                showCategories: `http://apis.france24.com/products/get_product/590c1202-2bfe-4cfc-9195-dd6781427c1f?token_application=${this._keys.token}`,
	                showContent: `http://apis.france24.com/products/get_product/ff377c90-793f-11e8-858f-005056a945c4?token_application=${this._keys.token}`,
	                mostViewed: `http://apis.france24.com/products/get_product/316d9842-b695-11e8-baad-005056bf3e3d?token_application=${this._keys.token}`,

	                latestStories: `http://apis.france24.com/products/get_product/823e1c24-73d3-11e8-86c2-005056bf3e3d?token_application=${this._keys.token}`,
	                liveTvPrograms: `http://apis.france24.com/products/get_product/7c04e322-7538-11e8-8034-005056bf3e3d?token_application=${this._keys.token}`,
	            },

	            es:{
	                latestBulletins: `http://apis.france24.com/products/get_product/ott_last_news_es?token_application=${this._keys.token}`,
	                showCategories: `http://apis.france24.com/products/get_product/b512070e-b696-11e8-b7c9-005056a945c4?token_application=${this._keys.token}`,
	                showContent: `http://apis.france24.com/products/get_product/1aba81a6-7940-11e8-a6b7-005056bf3e3d?token_application=${this._keys.token}`,
	                mostViewed: `http://apis.france24.com/products/get_product/7c9f1f20-b695-11e8-bc75-005056bf3e3d?token_application=${this._keys.token}`,

	                latestStories: `http://apis.france24.com/products/get_product/5d783d46-7529-11e8-81fa-005056a945c4?token_application=${this._keys.token}`,
	                liveTvPrograms: `http://apis.france24.com/products/get_product/2988d61e-7531-11e8-a2f5-005056bf3e3d?token_application=${this._keys.token}`,
	            }
	        };
	        //Primary and backup feeds for different languages.
	        this._primaryFeeds = {
	            fr: 'http://f24hls-i.akamaihd.net/hls/live/221146/F24_FR_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_FR_HI_HLS/*~id=metrological~hmac=e148dbeaa2be02e787b5ba0b80eee8e393f6a29964a8ff05833f9eb5857eab0f',
	            en: 'http://f24hls-i.akamaihd.net/hls/live/221147/F24_EN_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_EN_HI_HLS/*~id=metrological~hmac=c5e66cd72896ac906d5f5b7856666d4730fcdb1879c787fb5357fceb5bc2a63f',
	            ar: 'http://f24hls-i.akamaihd.net/hls/live/221148/F24_AR_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_AR_HI_HLS/*~id=metrological~hmac=2090dda4a5426d2650707679a19151a6bebb5051fad306eaa7e333873ff5ffad',
	            es: 'http://f24hls-i.akamaihd.net/hls/live/520845/F24_ES_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_ES_HI_HLS/*~id=metrological~hmac=a9ad9d35a542eaa2f4e015b6f150f6f8661b833cfd621ad0bc2c3a129635de2e'
	        };

	        this._backupFeeds = {
	            fr: 'http://f24hls-i.akamaihd.net/hls/live/221146-b/F24_FR_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_FR_HI_HLS/*~id=metrological~hmac=e148dbeaa2be02e787b5ba0b80eee8e393f6a29964a8ff05833f9eb5857eab0f',
	            en: 'http://f24hls-i.akamaihd.net/hls/live/221147-b/F24_EN_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_EN_HI_HLS/*~id=metrological~hmac=c5e66cd72896ac906d5f5b7856666d4730fcdb1879c787fb5357fceb5bc2a63f',
	            ar: 'http://f24hls-i.akamaihd.net/hls/live/221148-b/F24_AR_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_AR_HI_HLS/*~id=metrological~hmac=2090dda4a5426d2650707679a19151a6bebb5051fad306eaa7e333873ff5ffad',
	            es: 'http://f24hls-i.akamaihd.net/hls/live/520845-b/F24_ES_HI_HLS/master.m3u8?hdnts=exp=1609455599~acl=/hls/live/*/F24_ES_HI_HLS/*~id=metrological~hmac=a9ad9d35a542eaa2f4e015b6f150f6f8661b833cfd621ad0bc2c3a129635de2e'
	        };
	    }

	    _send(url) {
	        return fetch(Utils.proxyUrl(url)).then(r => r.json());
	    }

	    /**
	     * Returns the primary feed of the live tv.
	     * @returns {string} primary feed of the live tv
	     */
	    getPrimaryFeed() {
	        let countryCode = this._getCountryCode();
	        return this._primaryFeeds[countryCode];
	    }

	    getBackupFeed() {
	        let countryCode = this._getCountryCode();
	        return this._backupFeeds[countryCode];
	    }

	    /**
	     * Returns a promise to fetch the latest news bulletins.
	     *
	     * @returns {Promise}
	     */
	    getLatestBulletins() {
	        return this._send(this._endpoints[this._getCountryCode()].latestBulletins).then((data) => {
	            if(data.result == null || data.result == undefined) {
	                return Promise.reject('Get latest news bulletins returned no data');
	            }
	            let channels = data.result.channels;
	            if (!channels.length) {
	                return Promise.reject('Get latest news bulletins returned no data');
	            }

	            let hasImage = item => {
	                return item.images && item.images.formats && item.images.formats.length;
	            };

	            let hasVideo = item => {
	                return item.videos && item.videos.length &&
	                    item.videos[0].formats && item.videos[0].formats.length && item.videos[0].formats[0].code !== 'hls_secure'
	                    && !Api.isEmpty(item.videos[0].formats[0].url);
	            };

	            return Promise.resolve(channels.filter(item => (hasImage(item) && hasVideo(item)))
	                .map(channel => MediaItem.fromBulletinItem(channel))
	            );
	        });
	    }

	    /**
	     * Returns a promise to fetch the list of most viewed shows.
	     *
	     * @returns {Promise}
	     */
	    getMostViewed() {
	        return this._send(this._endpoints[this._getCountryCode()].mostViewed).then((data) => {
	            if(data.result == null || data.result == undefined) {
	                return Promise.resolve([]);
	            }
	            let list = data.result.list;
	            if (!list.length) {
	                return Promise.resolve([]);
	            }

	            let hasImage = item => {
	                return item.images && item.images.formats && item.images.formats.length;
	            };

	            let hasVideo = item => {
	                return item.formats && item.formats.length && Api.filterMP4(item.formats).length;
	            };

	            return Promise.resolve(list.filter(item => (hasImage(item) && hasVideo(item)))
	                .map(item => MediaItem.fromMostViewedItem(item))
	            );
	        });
	    }

	    /**
	     * Returns a promise to fetch the latest stories.
	     * @returns {Promise}
	     */
	    getLatestStories() {
	        return this._getShowContent(this._endpoints[this._getCountryCode()].latestStories)
	    }

	    /**
	     * Returns a promise to fetch the list of show categories
	     *
	     * @returns {Promise}
	     */
	    getShowCategories() {
	        return this._send(this._endpoints[this._getCountryCode()].showCategories).then((data) => {
	            if (data.result == null || data.result == undefined) {
	                return Promise.resolve([])
	            }
	            let shows = data.result.list;
	            if (!shows.length) {
	                return Promise.resolve([])
	            }

	            let hasImage = item => {
	                return item.images && item.images.formats && item.images.formats.length;
	            };

	            return Promise.resolve(shows.filter(item => hasImage(item))
	                .map(show => MediaItem.fromShowCategoryItem(show))
	            );
	        });
	    }

	    /**
	     * Returns a promise to fetch the list of videos available for a show category.
	     * @param nid nid of the show
	     * @returns {Promise}
	     */
	    getShows(nid) {
	        return this._getShowContent(`${this._endpoints[this._getCountryCode()].showContent}&nid=${nid}`)
	    }


	    _getShowContent(url) {
	        return this._send(url).then((data) => {
	            if (data.result == null || data.result == undefined) {
	                return Promise.resolve([]);
	            }
	            let shows = data.result.list;
	            if (!shows.length) {
	                return Promise.resolve([]);
	            }

	            let hasImage = item => {
	                return item.images && item.images.formats && item.images.formats.length;
	            };

	            let hasVideo = item => {
	                return item.videos && item.videos.length && item.videos[0].formats
	                    && item.videos[0].formats.length && Api.filterMP4(item.videos[0].formats).length;
	            };

	            return Promise.resolve(shows.filter(item => (hasImage(item) && hasVideo(item)))
	                .map(show => MediaItem.fromShowContentItem(show))
	            );
	        });
	    }

	    static filterMP4(formats) {
	        return formats.filter(item => {
	            //Check whether the url in the item is available or not.
	            return item.code === 'MP4' && !Api.isEmpty(item.url);
	        });
	    }

	    static isEmpty(value){
	        return (!value || 0 === value.length)
	    }

	    /**
	     * Returns a promise to fetch the current & next live programs.
	     * @returns {Promise}
	     */
	    getLivePrograms() {
	        return this._send(this._endpoints[this._getCountryCode()].liveTvPrograms).then((data) => {
	            if (!data.result || !data.result.content ||
	                !data.result.content.program_guide || !data.result.content.program_guide.length ||
	                !data.result.content.program_guide[0].schedule || !data.result.content.program_guide[0].schedule.length ||
	                !data.result.content.starting_date ) {
	                return Promise.reject('Get live program returned no data');
	            }

	            //Fetch the date from the serve.
	            let serverDateFormat = new Date().toLocaleString("en-US", {timeZone: "Europe/Paris"});

	            let schedule = data.result.content.program_guide[0].schedule;
	            let programs = [];
	            //create programs with start time & end time
	            for(let index = 0; index < schedule.length - 1; index++) {
	                //time stamps are in Central European Standard Time
	                let startDate = new Date(serverDateFormat);
	                let endDate = new Date(serverDateFormat);

	                if(schedule[index] && schedule[index].start_at) {
	                    let value = schedule[index].start_at.split(':');
	                    if (value.length > 0) {
	                        startDate.setHours(value[0]);
	                        startDate.setMinutes(value[1]);
	                        startDate.setSeconds(value[2]);
	                    }
	                }

	                if(schedule[index+1] && schedule[index+1].start_at) {
	                    let value = schedule[index + 1].start_at.split(':');
	                    if (value.length > 0) {
	                        endDate.setHours(value[0]);
	                        endDate.setMinutes(value[1]);
	                        endDate.setSeconds(value[2]);
	                    }
	                }

	                let start_time = startDate.getTime();
	                let end_time = endDate.getTime();

	                programs.push({
	                    title: schedule[index].program_label || schedule[index].type,
	                    start_time: start_time,
	                    end_time: end_time
	                });
	            }

	            //current time
	            let currentTime = new Date(serverDateFormat).getTime();
	            // Date.parse(data.literalGmtDate);
	            //Find out the current & next programs from the available
	            for(let index = 0; index <= programs.length - 1; index++) {
	                if(currentTime >= programs[index].start_time && currentTime < programs[index].end_time) {
	                    return Promise.resolve({current: programs[index],
	                        next: index === programs.length -1 ? null : programs[index + 1]});
	                }
	            }

	            return Promise.resolve();
	        });
	    }

	    /**
	     * Returns country code mapped to the box.
	     * @returns country code of the box if available, default value is english.
	     */
	    _getCountryCode(){
	        let selectedLangInfo = Storage.get(Config.SELECTED_LANGUAGE);
	        return (selectedLangInfo && selectedLangInfo.countryCode) ? selectedLangInfo.countryCode : 'en';
	    }
	}

	Api.MOST_VIEWED = 'EmptyData';

	class PlayerButton extends Lightning.Component {

	    static _template() {
	        return {
	            w: 68, h: 68,
	            Icon: {x: 34, y:34, mount : 0.5 , w: 68, h: 68}
	        };
	    }

	    set icon(icon) {
	        this._icon = icon;
	        let color = this._focused ? this.colors.focused : this._isActive ? this.colors.enabled : this.colors.disabled;
	        this.tag("Icon").patch({src: icon, color: color});
	    }

	    set active(v) {
	        this._isActive = v;
	        let color = v ? (this._focused ? this.colors.focused : this.colors.enabled) : this.colors.disabled;
	        this.tag("Icon").patch({smooth: {color: color}});
	    }

	    get active() {
	        return this._isActive;
	    }

	    static _states() {
	        return [
	            class Selected extends this {
	                $enter() {
	                    this.tag("Icon").patch({smooth: {color: this.colors.focused}});
	                }
	                $exit() {
	                    let color = this._isActive ? this.colors.enabled : this.colors.disabled;
	                    this.tag("Icon").patch({smooth: {color: color}});
	                }
	            }
	        ]
	    }

	    _focus() {
	        this._focused = true;
	        this._setState("Selected");
	    }

	    _unfocus() {
	        this._focused = false;
	        this._setState("");
	    }

	    get colors() {
	        return {
	            disabled: 0xffe7e4e4 ,
	            enabled: 0xff909090 ,
	            focused: 0xff00a7e3
	        };
	    }

	}

	class ScrollingText extends Lightning.Component {

	    static _template() {
	        return {
	            w: w=>w, h: h=>h, clipping: true,
	            GradientStart: {
	                zIndex: 1, alpha: 0,
	                w: ScrollingText.OFFSET + 20, h: 40, rect: true, colorLeft: 0xfff3f7f5, colorRight: 0x1fffffff
	            },
	            GradientEnd: {
	                zIndex: 1, alpha: 0,
	                x: (w=> w-22), w: 22, h: 40, rect: true, colorLeft: 0x1fffffff, colorRight: 0xfff3f7f5
	            },
	            PrimaryTextWrapper: {
	                x: ScrollingText.OFFSET,
	                w: 900,
	                //TODO setting below returns this.w=0.
	                //w: (w=> w-ScrollingText.OFFSET),
	                PrimaryText: {
	                    alpha: 1,
	                    text: {maxLines: 1},
	                    y: -5
	                },
	            },
	            SecondaryText: {
	                alpha: 0, text: {maxLines: 1}, y:-5
	            },

	            transitions:{alpha:{duration:0.3}}
	        };

	    }

	    set options(v) {
	        this.patch({
	            PrimaryTextWrapper: {PrimaryText: {color: v.color, text: {fontSize: v.fontSize, fontFace: v.fontFace, fontStyle: v.fontStyle}}},
	            SecondaryText: {color: v.color, text: {fontSize: v.fontSize, fontFace: v.fontFace, fontStyle: v.fontStyle}}
	        });
	    }

	    set label(v){
	        this._reset();
	        this.patch({
	            PrimaryTextWrapper: {PrimaryText: {text: {text: v}}},
	            SecondaryText: {text: {text: v}}
	        });
	    }

	    _init() {
	        this.tag('PrimaryText').on('txUnloaded', () => {
	            this._reset();
	        });

	        this.tag('PrimaryText').on('txLoaded', () => {
	            this.patch({
	                PrimaryTextWrapper: {
	                    PrimaryText: {smooth: {x: [0, {duration: 0}]}},
	                },
	                SecondaryText: {smooth: {alpha: 0, x: this.w}},
	            });

	            const width = this.tag('PrimaryText').renderWidth;
	            if(width <= this.tag('PrimaryTextWrapper').finalW || this._primaryTextFirstAnimation) {
	                if(!this._primaryTextFirstAnimation) {
	                    this.patch({
	                        GradientStart: {smooth: {alpha: [0, {delay:0}]}},
	                        GradientEnd: {smooth: {alpha: 0}},
	                    });
	                }
	                return;
	            }

	            //Scroll 90% of the text first
	            let firstShift = (width - this.tag('PrimaryTextWrapper').finalW + ScrollingText.OFFSET * 0.9) ;
	            let firstDuration = firstShift / 150;
	            this._primaryTextFirstAnimation = this.tag('PrimaryText').animation({duration: firstDuration, delay: 3, repeat: 0, stopMethod: 'immediate', actions: [
	                    {p: 'x', v: {0: {v:0, sm:0}, 0.5: {v: -firstShift/2, sm: 0}, 1: {v: -firstShift, sm:0}}}
	                ]});

	            //Scroll 10% of the text next
	            let secondShift = this.tag('PrimaryTextWrapper').finalW + ScrollingText.OFFSET * 0.9;
	            let secondDuration = secondShift / 150;
	            this._primaryTextSecondAnimation = this.tag('PrimaryText').animation({duration: secondDuration, delay: 0, repeat: 0, stopMethod: 'immediate', actions: [
	                    {p: 'x', v: {0: {v:-firstShift, sm:0}, 0.5: {v: -(firstShift + secondShift/2), sm: 0}, 1: {v: - (firstShift + secondShift), sm:0}}}
	                ]});

	            //Scroll secondary text
	            let duration = (this.tag('PrimaryTextWrapper').finalW ) / 150;
	            this._secondaryTextAnimation = this.tag('SecondaryText').animation({duration: duration, delay: 0.6, repeat: 0, stopMethod: 'immediate', actions: [
	                    {p: 'x', v: {0: {v:this.w, sm:0}, 0.5: {v: this.w/2 + ScrollingText.OFFSET/2, sm: 0}, 1: {v: ScrollingText.OFFSET, sm:0}}}
	                ]});

	            //After the 90% of the text is scrolled then start scrolling secondary text
	            this._primaryTextFirstAnimation.on('finish', () => {
	                this._primaryTextFirstAnimation = null;
	                if(this._primaryTextSecondAnimation) {
	                    this._primaryTextSecondAnimation.start();
	                }
	                this.tag('SecondaryText').setSmooth('alpha', 1);
	                if(this._secondaryTextAnimation) {
	                    this._secondaryTextAnimation.start();
	                }
	            });


	            this._primaryTextFirstAnimation.on('start', () => {
	                this.patch({
	                    GradientStart: {smooth: {alpha: [1, {delay :3}]}},
	                    GradientEnd: {smooth: {alpha: 1}}
	                });
	            });


	            this._secondaryTextAnimation.on('finish', () => {
	                this.patch({
	                    GradientStart: {smooth: {alpha: [0, {delay :0}]}},
	                    GradientEnd: {smooth: {alpha: 1}}
	                });
	            });

	            this.fireAncestors('$timeout', (ScrollingText.SCROLL_START_DELAY + firstDuration + duration + ScrollingText.SCROLL_END_DELAY) * 1000);
	            this._primaryTextFirstAnimation.start();
	        });
	    }

	    _reset() {

	        //Reset the gradient alpha value.
	        this.patch({
	            GradientStart: {smooth: {alpha: [0, {delay :0}]}},
	            GradientEnd: {smooth: {alpha: 1}}
	        });

	        if(this._secondaryTextAnimation) {
	            this._secondaryTextAnimation.stop();
	            this._secondaryTextAnimation = null;
	        }

	        if(this._primaryTextSecondAnimation)  {
	            this._primaryTextSecondAnimation.stop();
	            this._primaryTextSecondAnimation = null;
	        }

	        if(this._primaryTextFirstAnimation) {
	            this._primaryTextFirstAnimation.stop();
	            this._primaryTextFirstAnimation = null;
	        }
	    }
	}

	ScrollingText.OFFSET = 26;
	ScrollingText.SCROLL_START_DELAY = 2;
	ScrollingText.SCROLL_END_DELAY = 3;

	class PlayerProgress extends Lightning.Component {

	    static _template() {
	        return {
	            Progress: {
	                Total: {
	                    texture: Lightning.Tools.getRoundRect(1260, 7, 0), color: 0xffb2b2b2
	                },
	                Active: {color: 0xff00a7e3},
	            }
	        };
	    }

	    set _progress(v) {
	        const now = Date.now();
	        let estimation = 0;
	        if (!this._last || (this._last < now - 1000)) {
	            estimation = 500;
	        } else {
	            estimation = now - this._last;
	        }
	        this._last = now;
	        const x = v * 1260;

	        estimation *= 0.001;
	        this.tag("Total").setSmooth('x', x, {timingFunction: 'linear', duration: estimation});
	        this.tag("Total").setSmooth('texture.x', x, {timingFunction: 'linear', duration: estimation});
	        this.tag("Active").setSmooth('texture.w', Math.max(x, 0.0001) /* force clipping */, {
	            timingFunction: 'linear',
	            duration: estimation
	        });
	    }

	    setProgress(currentTime, duration) {
	        this._progress = currentTime / Math.max(duration, 1);
	    }

	    _init() {
	        this.tag("Active").texture = {
	            type: Lightning.textures.SourceTexture,
	            textureSource: this.tag("Total").texture.source
	        };
	    }

	}

	class PlayerControls extends Lightning.Component {

	    static _template() {
	        return {
	            Shadow: {
	                x: -35, y: -30,
	                color: 0xB2000000,
	            },
	            Background: {
	                x: 0, y: 0, h: 110, w: 1260, color: 0xfff3f7f5, rect: true
	            },
	            RootFlex: {
	                x: 30, y: 23,
	                flex: {direction: 'row'},
	                Buttons: {
	                    flexItem: {},
	                    flex: {direction: 'row', justifyItems: 'flex-start'},
	                    Previous: {
	                        flexItem: {},
	                        type: PlayerButton, icon: Utils.asset('images/rewind.png')
	                    },
	                    Play: {
	                        flexItem: {marginLeft: 20},
	                        type: PlayerButton, icon: Utils.asset('images/pause.png')
	                    },
	                    Next: {
	                        flexItem: {marginLeft: 20},
	                        type: PlayerButton, icon: Utils.asset('images/forward.png')}
	                },
	                Metadata: {
	                    flexItem: {},
	                    TitleFlex: {
	                        x: 56,
	                        y: -8,
	                        flex: {direction: "row", alignItems: "flex-end"},
	                        Title: {flexItem: {},
	                            text: {fontSize: 32, lineHeight: 45, wordWrapWidth: 840, fontFace: "OpenSans", fontStyle: '600'}, color: 0xff101010,
	                        },
	                        Duration: {flexItem: {marginBottom: 18, marginLeft: 15},
	                            text: {fontSize: 24, fontFace: "OpenSans", fontStyle: '600'}, color: 0xff909090}
	                    },
	                    Description: {
	                        x: 30, y: 40, w: 926, alpha: 0,
	                        type: ScrollingText, options: {fontSize: 28, fontFace: "OpenSans", fontStyle: '600', color: 0xff101010}}
	                }
	            },
	            Progress: {
	                x: -1, y: 109, type: PlayerProgress, alpha: 0
	            }
	        };
	    }

	    showButtons(previous, next) {
	        this._showButtons(previous, next);
	    }

	    setProgress(currentTime, duration) {
	        this.tag("Progress").setProgress(currentTime, duration);
	    }

	    _init() {
	        this.tag('Title').on('txLoaded', () => {
	            let backgroundY = 0, backgroundH = 110, shadowY = -30, shadowH = this._getState('Recorded') ? 127 : 120;
	            let noOfLines = this.tag('Title').texture._source.renderInfo.lines.length;
	            if(noOfLines >= 2) {
	                let titleY;
	                if(noOfLines === 2) {
	                    titleY = -8;
	                } else {
	                    shadowY = shadowY - (PlayerControls.TITLE_ONE_LINE_HEIGHT * (noOfLines - 2));
	                    shadowH = shadowH + (PlayerControls.TITLE_ONE_LINE_HEIGHT * (noOfLines - 2));
	                    titleY = -8 - (PlayerControls.TITLE_ONE_LINE_HEIGHT * (noOfLines - 2));
	                    backgroundY = backgroundY - PlayerControls.TITLE_ONE_LINE_HEIGHT * (noOfLines - 2);
	                    backgroundH = backgroundH + (PlayerControls.TITLE_ONE_LINE_HEIGHT * (noOfLines - 2));
	                }

	                this.tag('Shadow').patch({
	                    smooth: {y: shadowY}
	                });
	                this.tag("Shadow").texture = Lightning.Tools.getShadowRect(1270, shadowH, 0, 30, 40);

	                this.tag('Background').patch({
	                    smooth: {y: backgroundY, h: backgroundH}
	                });

	                this.tag('RootFlex').patch({
	                    Metadata: {
	                        TitleFlex : {
	                            smooth: {y: titleY}
	                    },
	                    Description: {smooth: {alpha: 0}}
	                }
	              });
	           } else {
	                this.tag('Shadow').patch({
	                    smooth: {y: shadowY}
	                });
	                this.tag("Shadow").texture = Lightning.Tools.getShadowRect(1270, shadowH, 0, 30, 40);

	                this.tag('Background').patch({
	                    smooth: {y: backgroundY, h: backgroundH}
	                });

	               if(this._description) {
	                   this.tag('RootFlex').patch({
	                       Metadata: {
	                           TitleFlex : {
	                               smooth: {y: -8}
	                           },
	                           Description: {smooth: {alpha: this._description === 'Live' ? 0.5 : 1}}
	                       }
	                   });
	               } else {
	                   this.tag('RootFlex').patch({
	                       Metadata: {
	                           TitleFlex : {
	                               smooth: {y: 12}
	                           },
	                           Description: {smooth: {alpha: 0}}
	                       }
	                   });
	               }
	           }
	        });
	    }

	    set text({title= null, description= null}) {
	        this._description = description;
	        if(description) {
	            this.tag("Description").label = description;
	        }
	        this.tag("Title").text = title || ' ';
	    }

	    set duration(duration) {
	        this.tag("Duration").text = Math.floor(duration / 60) === 0 ? "" : Player.formatTime(duration);
	    }

	    get duration() {
	        return this.tag("Duration").text.text;
	    }

	    set mode(mode) {
	        if(mode === Player.MODES.RECORDED) {
	            //TODO Check whether there is any way to force call the $enter of a state.
	            this._setState("");
	            this._setState("Recorded");
	        } else {
	            this._setState("Live");
	        }
	    }

	    set paused(v) {
	        this.tag("Play").icon = v ? Utils.asset('images/play.png') : Utils.asset('images/pause.png');    }

	    _changedState({newState}) {
	        this.tag("Previous").visible = newState === "Live" ? false : true;
	        this.tag("Next").visible = newState === "Live" ? false : true;
	        this.tag("Duration").setSmooth('alpha', newState === "Live" ? 0 : 1);
	    }

	    static _states() {
	        return [
	            class Recorded extends this {

	                $enter() {
	                    this._setState("Recorded.Play");
	                    this.showButtons(false, false);
	                    this.tag('Progress').setSmooth('alpha', 1);
	                }

	                _showButtons(previous, next) {
	                    let buttons = [];
	                    if (previous) buttons.push("Previous");
	                    buttons = buttons.concat("Play");
	                    if (next) buttons.push("Next");
	                    this._setActiveButtons(buttons);
	                }

	                get _activeButtonIndex() {
	                    let button = this.tag("Buttons").getByRef(this._getViewRef(this._getState()));
	                    if (!button.active) {
	                        button = this.tag("Play");
	                    }
	                    return this._activeButtons.indexOf(button);
	                }

	                get _activeButton() {
	                    return this._activeButtons[this._activeButtonIndex];
	                }

	                _setActiveButtons(buttons) {
	                    this._activeButtons = [];
	                    this.tag("Buttons").children.map(button => {
	                        button.active = (buttons.indexOf(button.ref) !== -1);
	                        if (button.active) {
	                            this._activeButtons.push(button);
	                        }
	                    });

	                    this._checkActiveButton();
	                }

	                _checkActiveButton() {
	                    // After changing the active buttons, make sure that an active button is selected.
	                    let index = this._activeButtonIndex;
	                    if (index === -1) {
	                        if (this._index >= this._activeButtons.length) {
	                            this._index = this._activeButtons.length - 1;
	                        }
	                    }
	                    this._setState("Recorded." + this._activeButtons[index].ref);
	                }

	                _handleLeft() {
	                    let index = this._activeButtonIndex;
	                    if (index > 0) {
	                        index--;
	                    }
	                    this._setState("Recorded." + this._activeButtons[index].ref);
	                }

	                _handleRight() {
	                    let index = this._activeButtonIndex;
	                    if (index < this._activeButtons.length - 1) {
	                        index++;
	                    }
	                    this._setState("Recorded." + this._activeButtons[index].ref);
	                }

	                _handleEnter() {
	                    this.signal('press' + this._activeButton.ref);
	                }

	                _getFocused() {
	                    return this.tag(this._getViewRef(this._getState()));
	                }

	                _getViewRef(state) {
	                    return state.split('.')[1];
	                }

	                static _states() {
	                    return [
	                        class Previous extends this {
	                        },
	                        class Play extends this {
	                        },
	                        class Next extends this {
	                        }
	                    ];
	                }
	            },
	            class Live extends this {

	                $enter() {
	                    this.tag('Progress').setSmooth('alpha', 0);
	                }

	                _handleEnter() {
	                    this.signal('pressPlay');
	                }

	                _getFocused() {
	                    return this.tag("Play");
	                }

	            }
	        ];
	    }

	}

	PlayerControls.TITLE_ONE_LINE_HEIGHT = 45;

	class Player extends Lightning.Component {

	    static _template() {
	        return {
	            zIndex: 99,
				MediaPlayer: {type: Mediaplayer},
	            Loader: {
	                w: 1920, h: 1080, rect: true, color: Config.COLORS.BACKGROUND, alpha: 0,
	                Spinner: {x: 960, y: 540, mount: 0.5, type: SpinningLogo},
	                transitions: {alpha: {duration: 0.3}}
	            },

	            MiniLoader: {
	                x:93, y:181, w:715, h:402, rect: true, color: Config.COLORS.BACKGROUND, alpha: 0,
	                Spinner: {x:350, y:197, mount: 0.5, type: SpinningLogo},
	                transitions: {alpha: {duration: 0.3}}
	            },

	            Controls: {
	                x: 330, y: 903, alpha: 0,
	                type: PlayerControls, signals: {pressPlay: true, pressPrevious: true, pressNext: "_pressNext"},
	                transitions: {alpha: {duration: 0.3}}
	            },
	        };
	    }

	    _init() {
	        this._api = this.fireAncestors('$getApi');
	        this.tag('MediaPlayer').updateSettings({consumer:this});
	    }

	    /**
	     * Plays the given recorded video play list.
	     * @param item first item to be played
	     * @param items playlist
	     * @returns {boolean}
	     */
	    play({item, items}) {
	        this._items = items;
	        this._item = item;
	        this._setState("Fullscreen.Recorded.Loading");
	        return this._play();
	    }

	    /**
	     * Plays the live streaming.
	     * @param minimized true to indicate the player should play in window mode.
	     * @returns {boolean}
	     */
	    playLive(title, minimized) {
	        const api = this.fireAncestors('$getApi');
	        this._item = {title: title, description: 'Live', stream : {link: api.getBackupFeed()}};
	        if(minimized) {
	            this._setState("Minimized");
	        } else {
	            this.tag('MediaPlayer').pause();
	            if(this._stream && this._stream.link === api.getPrimaryFeed()){
	                this._item = {title: title, description: 'Live', stream : {link: api.getBackupFeed()}};
	            }else{
	                this._item = {title: title, description: 'Live', stream : {link: api.getPrimaryFeed()}};
	            }

	            this._setState("Fullscreen.Live.Loading");
	        }
	        return this._play();
	    }

	    static formatTime(seconds) {
	        return Math.floor(seconds / 60) + " min";
	    }

	    _handleBack() {
	        this.signal('playerStop');
	        this._setState("Inactive");
	    }

	    static _states() {
	        return [
	            class Fullscreen extends this {

	                $enter() {
	                    this._setInterfaceTimeout();
	                }

	                $exit() {
	                    clearTimeout(this._timeout);
	                }

	                $timeout(duration) {
	                    this._duration = duration;
	                    this._durationStart = Date.now();
	                    this._setInterfaceTimeout();
	                }

	                pressPlay() {
	                    this.tag('MediaPlayer').playPause();
	                }

	                _playerStop() {
	                    this.signal('playerStop');
	                    this._setState("Inactive");
	                }

	                $mediaplayerPause() {
	                    this.tag("Controls").paused = true;
	                }
	                _handlePlay() {
	                    this.tag('MediaPlayer').doPlay();
	                }
	                _handlePause() {
	                    this.tag('MediaPlayer').doPause();
	                }
	                _handlePlayPause() {
	                    this.tag('MediaPlayer').playPause();
	                }
	                _handleStop() {
	                    this.tag('MediaPlayer').close();
	                }
	                _handleRewind() {
	                    //No functionality for this app
	                }
	                _handleForward() {
	                    //No functionality for this app
	                }
	                $mediaplayerPlay() {
	                    this.tag("Controls").paused = false;
	                }

	                $mediaplayerStop() {
	                   this._playerStop();
	                }

	                $mediaplayerError(){
	                    this._setState("Inactive");
	                    this.signal('playerError');
	                }

	                _handleBack() {
	                    this.tag('MediaPlayer').close();
	                    this._playerStop();
	                }

	                _captureKey() {
	                    this._setInterfaceTimeout();
	                    return false;
	                }

	                _setInterfaceTimeout() {
	                    if (this._timeout) {
	                        clearTimeout(this._timeout);
	                    }

	                    let timeout = (this._duration && (this._durationStart + this._duration) - Date.now()) > 8000 ? this._duration : 8000;
	                    this._timeout = setTimeout(() => {
	                        this._hide();
	                    }, timeout);
	                }

	                _getFocused() {
	                    return this.tag("Controls");
	                }

	                _unfocus(){
	                    //Close the running stream before playing another
	                    // this.tag('MediaPlayer').close();
	                }

	                static _states() {
	                    return [
	                        class Recorded extends this {
	                            _play() {
	                                this.tag('MediaPlayer').updateSettings({consumer:this, videoPos:[0,0,1920,1080]});
	                                this.tag("Controls").mode = Player.MODES.RECORDED;
	                                this._setItem(this._item);
	                                return !!this._stream;
	                            }

	                            _setItem(item) {
	                                this._setState('Fullscreen.Recorded.Loading');
	                                this._duration = 0;
	                                this._setInterfaceTimeout();
	                                this._stream = item.stream;
	                                this.tag("Controls").text = {title: item.title.toUpperCase(), description: (item.title === item.description) ? '' : item.description};
	                                this.tag("Controls").duration = 0;

	                                this.tag("Controls").setProgress(0, 0);
	                                this._index = this._items.indexOf(item);
	                                this.tag("Controls").showButtons(this._index > 0, this._index < this._items.length - 1);

	                                this.tag('MediaPlayer').open(this._stream.link);
	                            }

	                            pressPrevious() {
	                                const index = this._index - 1;
	                                if (index < 0) {
	                                    this._index = this._items.length - 1;
	                                }
	                                this._setItem(this._items[index]);
	                            }

	                            _pressNext() {
	                                if (!this._items.length) {
	                                    this._playerStop();
	                                }
	                                const index = (this._index + 1) % this._items.length;
	                                this._setItem(this._items[index]);
	                            }

	                            $mediaplayerProgress({currentTime, duration}) {
	                                if(!this.tag("Controls").duration) this.tag("Controls").duration = duration;
	                                this.tag("Controls").setProgress(currentTime, duration);
	                            }

	                            $mediaplayerEnded() {
	                                this._pressNext();
	                            }

	                            _hide() {
	                                this._setState("Fullscreen.Recorded.Hidden");
	                            }

	                            static _states() {
	                                return [
	                                    class Hidden extends this {
	                                        $enter({prevState}) {
	                                            this._prevState = prevState;
	                                            this.tag('Controls').setSmooth('alpha', 0);
	                                        }
	                                        $exit() {
	                                            this._duration = 0;
	                                            this._setInterfaceTimeout();
	                                            this.tag('Controls').setSmooth('alpha', 1);
	                                        }
	                                        _captureBack(){
	                                            this._setState(this._prevState);

	                                        }
	                                        _captureEnter(){
	                                            this._setState(this._prevState);
	                                        }
	                                        _captureLeft(){
	                                            this._setState(this._prevState);
	                                        }
	                                        _captureRight(){
	                                            this._setState(this._prevState);
	                                        }
	                                        _captureKey(){
	                                            this._setState(this._prevState);

	                                            return false;
	                                        }

	                                    },
	                                    class Loading extends this {
	                                        $enter() {
	                                            this.tag('Loader').setSmooth('alpha', 1);
	                                            this.tag('Controls').setSmooth('alpha', 0);
	                                        }

	                                        $exit() {
	                                            this.tag('Controls').setSmooth('alpha', 1);
	                                            this.tag('Loader').setSmooth('alpha', 0);
	                                        }

	                                        $mediaplayerPlay() {
	                                            this.tag("Controls").paused = false;
	                                            this._setState('Fullscreen.Recorded');
	                                        }

	                                        _getFocused() {
	                                            return this.tag('Spinner');
	                                        }

	                                        _hide() {
	                                            this._setInterfaceTimeout();
	                                        }
	                                    }
	                                ];
	                            }
	                        },
	                        class Live extends this {
	                            $enter() {
	                                this.tag('Controls').setSmooth('alpha', 1);
	                                this.tag('Loader').setSmooth('alpha', 0);

	                                this._setTimeout(0);
	                            }

	                            $exit() {
	                                this._clearTimeout();
	                                //reset the live program information
	                                this._setLiveProgramInfo();
	                            }

	                            _setTimeout(duration) {
	                                this._timerId = setTimeout(() => {
	                                    this._api.getLivePrograms().then( (programs) => {
	                                        //set the live program information
	                                        this._setLiveProgramInfo(programs);

	                                        let nextPingAt = Config.LIVE_PROGRAMS_POLL_FREQUENCY;
	                                        if(programs) {
	                                            var serverTime = new Date().toLocaleString("en-US", {timeZone: "Europe/Paris"});
	                                            let currentTime = new Date(serverTime).getTime();
	                                            let nextProgramIn = programs.current.end_time - currentTime;
	                                            //If the next program starts in less than the poll frequency then adjust the next ping
	                                            if(nextProgramIn < Config.LIVE_PROGRAMS_POLL_FREQUENCY) {
	                                                nextPingAt = nextProgramIn;
	                                            }
	                                        }
	                                        this._setTimeout(nextPingAt);
	                                    }).catch(err => {
	                                        console.error(err);
	                                        this._clearTimeout();
	                                    });
	                                }, duration);
	                            }

	                            _clearTimeout() {
	                                clearTimeout(this._timerId);
	                            }

	                            _setLiveProgramInfo(programs) {
	                                if(programs) {
	                                    this.tag("Controls").text = {title: programs.current.title.toUpperCase(), description: this._item.description};
	                                } else {
	                                    this.tag("Controls").text = {description: this._item.description};
	                                }
	                            }

	                            _hide() {
	                                this._setState("Fullscreen.Live.Hidden");
	                            }

	                            _handleBack(){
	                                this.signal('playMinimized');
	                                this.tag('Controls').setSmooth('alpha', 0);
	                                this.tag("Controls").setProgress(0, 0);
	                            }

	                            static _states() {
	                                return [
	                                    class Hidden extends Live {
	                                        $enter() {
	                                            this.tag('Controls').setSmooth('alpha', 0);
	                                        }

	                                        $exit() {
	                                            this._setInterfaceTimeout();
	                                            this.tag('Controls').setSmooth('alpha', 1);
	                                        }

	                                        _captureBack(){
	                                            this._setState('Fullscreen.Live');
	                                        }
	                                        _captureEnter(){
	                                            this._setState('Fullscreen.Live');
	                                        }
	                                        _captureKey(){
	                                            this._setState('Fullscreen.Live');
	                                            return false;
	                                        }
	                                    },
	                                    class Loading extends Live {
	                                        $enter() {
	                                            this.tag('Loader').setSmooth('alpha', 1);
	                                            this.tag('Controls').setSmooth('alpha', 0);
	                                        }

	                                        $exit() {
	                                            this.tag('Controls').setSmooth('alpha', 1);
	                                            this.tag('Loader').setSmooth('alpha', 0);
	                                        }

	                                        _play() {
	                                            this._setInterfaceTimeout();
	                                            this.tag("Controls").mode = Player.MODES.LIVE;
	                                            this._stream = this._item.stream;
	                                            this.tag("Controls").text = {title: this._item.title, description: this._item.description};
	                                            this.tag('MediaPlayer').updateSettings({consumer:this, videoPos:[0,0,1920,1080]});
	                                            this.tag('MediaPlayer').open(this._stream.link);
	                                            return this._stream;
	                                        }

	                                        $mediaplayerPlay() {
	                                            this.tag("Controls").paused = false;
	                                            this._setState('Fullscreen.Live');
	                                        }

	                                        _getFocused() {
	                                            return this.tag('Spinner');
	                                        }

	                                        _hide() {
	                                            this._setInterfaceTimeout();
	                                        }
	                                    }
	                                ];
	                            }
	                        }
	                    ];
	                }
	            },

	            class Minimized extends this {
	                $enter({}, args) {
	                    this.tag('MiniLoader').setSmooth('alpha', 1);
	                    if(args && args.play){
	                        this._play();
	                    }
	                }

	                $exit(){
	                    this.tag('MiniLoader').setSmooth('alpha', 0);
	                }

	                $mediaplayerPlay() {
	                    this.tag('MiniLoader').setSmooth('alpha', 0);
	                }

	                closePlayer(){
	                    this._setState('Pause');
	                }

	                _play() {
	                    //Check whether the last played stream is live stream or not.
	                    if(this._stream && (this._stream.link === this._api.getPrimaryFeed() || this._stream.link === this._api.getBackupFeed())){
	                        this.tag('MediaPlayer').updateSettings({consumer:this, videoPos:[61,121,478,268]}); //88, 180, 815, 585
	                        this.tag('MediaPlayer').doPlay();
	                    }else {
	                        this.tag('MediaPlayer').close();
	                        this._stream = this._item.stream;
	                        this.tag('MediaPlayer').open(this._stream.link, {videoPosition:[61,121,478,268]}); // 88, 180, 815, 585
	                    }

	                    if(this.tag('MediaPlayer').isPlaying()){
	                        this.tag('MiniLoader').setSmooth('alpha', 0);
	                    }else{
	                        this.tag('MiniLoader').setSmooth('alpha', 1);
	                    }
	                    return true;
	                }
	            },

	            class Pause extends this {
	                $enter() {
	                    this.tag('MediaPlayer').doPause();
	                }

	                $exit(){
	                    this.tag('MediaPlayer').doPlay();
	                }
	            },

	            class Inactive extends this {
	                $enter() {
	                    this.tag('Controls').setSmooth('alpha', 0);
	                    this.tag("Controls").setProgress(0, 0);
	                }
	            }
	        ]
	    }
	}

	Player.MODES = {
	    RECORDED : 'recorded',
	    LIVE : 'live'
	};

	class App extends Lightning.Component {

	    static getLocale() {
	        let selectedLanguage = Storage.get(Config.SELECTED_LANGUAGE);
	        let countryCode = selectedLanguage ? selectedLanguage.countryCode : 'en';
	        Locale$1.setLanguage(countryCode);
	        return Utils.asset('locale/locale.json')
	    }


	    static getFonts() {
	        return [
	            {family: 'OpenSans', url: Utils.asset('fonts/OpenSans-Regular.ttf'), descriptors: {}},
	            {family: 'OpenSans', url: Utils.asset('fonts/OpenSans-SemiBold.ttf'), descriptors: {'weight': '600'}},
	            {family: 'OpenSans', url: Utils.asset('fonts/OpenSans-Bold.ttf'), descriptors: {'weight': 'bold'}},

	            {family: 'Amri', url: Utils.asset('fonts/Amiri-Bold.ttf'), descriptors: {}},
	            ]
	    }

	    static _template() {
	        return {
	            Splash: {type: Splash},
	            Player: {type: Player, alpha: 0, signals: {playerStop: true, playMinimized:true, playerError:true}},
	            Error:{alpha:0, zIndex:2, type:Alert, signals:{OnClick:true}},
	            transitions:{alpha:{duration:0.3}}
	        };
	    }

	    _construct() {
	        this._api = new Api();
	    }

	    $getApi() {
	        return this._api;
	    }

	    _init() {
	        this._setState("Splash");
	    }

	    static _states() {
	        return [
	            class Splash extends this {

	                $exit() {
	                    this.tag("Splash").setSmooth('alpha', 0);
	                }

	                _getFocused() {
	                    return this.tag("Splash");
	                }
	                $onReady() {
	                    this.patch({Main: {
	                        type: Main, alpha: 0, transitions: {alpha: {duration:0.3, delay: 0.1}}
	                    }});
	                }
	                $hideSplash() {
	                    this._setState("Main");
	                }
	            },
	            class Main extends this {

	                $enter() {
	                    this.tag("Main").setSmooth('alpha', 1);
	                }

	                $exit() {
	                    this.tag("Main").setSmooth('alpha', 0);
	                }

	                _getFocused() {
	                    return this.tag("Main");
	                }

	                $play({item, items}) {
	                    const player = this.tag('Player');
	                    const playlist = {item: item.getMediaplayerItem(), items: items.map(item => item.getMediaplayerItem())};
	                    if (player.play(playlist)) {
	                        this._setState("Playing");
	                    }
	                }

	                $playLive(title, minimized) {
	                    const player = this.tag('Player');
	                    if (player.playLive(title, minimized)) ;

	                    if(minimized) {
	                        this._setState("Main.MinimizedPlayback");
	                    } else {
	                        this._setState("Playing");
	                    }
	                }

	                static _states() {
	                    return [
	                        class MinimizedPlayback extends this {
	                            $enter() {
	                                this.tag("Player").setSmooth('alpha', 1);
	                            }
	                            $exit() {
	                                this.tag("Player").setSmooth('alpha', 0);
	                            }

	                            $stopMinimizedPlayback() {
	                                this.tag('Player').closePlayer();
	                                this._setState("Main");
	                            }
	                        }
	                    ];
	                }
	            },
	            class Playing extends this {
	                $enter() {
	                    this.tag("Player").setSmooth('alpha', 1);
	                }
	                $exit() {
	                    this.tag("Player").setSmooth('alpha', 0);
	                }

	                playerStop() {
	                    // Last item has been fully played.
	                    this._setState("Main");
	                }

	                playMinimized(){
	                    this._setState("Main.MinimizedPlayback");
	                }
	                playerError(){
	                    this._setState("Error");
	                }
	                _getFocused() {
	                    return this.tag("Player");
	                }
	            },

	            class Error extends this {
	                $enter() {
	                    this.tag("Main").setSmooth('alpha', 0.5);
	                    this.tag("Error").setSmooth('alpha', 1);
	                    this.tag('Error').description = Locale$1.tr.mediaError;
	                }
	                $exit() {
	                    this.tag("Error").setSmooth('alpha', 0);
	                    this.tag("Main").setSmooth('alpha', 1);
	                }

	                _getFocused(){
	                    return this.tag('Error')
	                }

	                OnClick(){
	                    this._setState("Main");
	                }
	            }
	        ];
	    }
	}

	function index () {
		return Launch(App, ...arguments)
	}

	return index;

}());
